@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2013-2014, 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2008-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,34 +39,34 @@
 #include "ShadowChickenInlines.h"
 #include "StackVisitor.h"
 #include "StrongInlines.h"
+#include "VMEntryScopeInlines.h"
 
 namespace JSC {
 
 class LineAndColumnFunctor {
 public:
-    StackVisitor::Status operator()(StackVisitor& visitor) const
+    IterationStatus operator()(StackVisitor& visitor) const
     {
-        visitor->computeLineAndColumn(m_line, m_column);
-        return StackVisitor::Done;
+        m_lineColumn = visitor->computeLineAndColumn();
+        return IterationStatus::Done;
     }
 
-    unsigned line() const { return m_line; }
-    unsigned column() const { return m_column; }
+    unsigned line() const { return m_lineColumn.line; }
+    unsigned column() const { return m_lineColumn.column; }
 
 private:
-    mutable unsigned m_line { 0 };
-    mutable unsigned m_column { 0 };
+    mutable LineColumn m_lineColumn;
 };
 
 Ref<DebuggerCallFrame> DebuggerCallFrame::create(VM& vm, CallFrame* callFrame)
 {
-    if (UNLIKELY(!callFrame)) {
+    if (!callFrame) [[unlikely]] {
         ShadowChicken::Frame emptyFrame;
         RELEASE_ASSERT(!emptyFrame.isTailDeleted);
         return adoptRef(*new DebuggerCallFrame(vm, callFrame, emptyFrame));
     }
 
-    if (callFrame->isDeprecatedCallFrameForDebugger()) {
+    if (callFrame->isEmptyTopLevelCallFrameForDebugger()) {
         ShadowChicken::Frame emptyFrame;
         RELEASE_ASSERT(!emptyFrame.isTailDeleted);
         return adoptRef(*new DebuggerCallFrame(vm, callFrame, emptyFrame));
@@ -111,18 +111,9 @@ RefPtr<DebuggerCallFrame> DebuggerCallFrame::callerFrame()
     return m_caller;
 }
 
-JSGlobalObject* DebuggerCallFrame::globalObject()
+JSGlobalObject* DebuggerCallFrame::globalObject(VM& vm)
 {
-    return scope()->globalObject();
-}
-
-JSC::JSGlobalObject* DebuggerCallFrame::deprecatedVMEntryGlobalObject() const
-{
-    ASSERT(isValid());
-    if (!isValid())
-        return nullptr;
-    VM& vm = m_validMachineFrame->deprecatedVM();
-    return vm.deprecatedVMEntryGlobalObject(m_validMachineFrame->lexicalGlobalObject(vm));
+    return scope(vm)->globalObject();
 }
 
 SourceID DebuggerCallFrame::sourceID() const
@@ -135,37 +126,35 @@ SourceID DebuggerCallFrame::sourceID() const
     return sourceIDForCallFrame(m_validMachineFrame);
 }
 
-String DebuggerCallFrame::functionName() const
+String DebuggerCallFrame::functionName(VM& vm) const
 {
     ASSERT(isValid());
     if (!isValid())
         return String();
 
-    VM& vm = m_validMachineFrame->deprecatedVM();
     if (isTailDeleted()) {
-        if (JSFunction* func = jsDynamicCast<JSFunction*>(vm, m_shadowChickenFrame.callee))
+        if (JSFunction* func = jsDynamicCast<JSFunction*>(m_shadowChickenFrame.callee))
             return func->calculatedDisplayName(vm);
-        return m_shadowChickenFrame.codeBlock->inferredName().data();
+        return String::fromLatin1(m_shadowChickenFrame.codeBlock->inferredName().data());
     }
 
     return m_validMachineFrame->friendlyFunctionName();
 }
 
-DebuggerScope* DebuggerCallFrame::scope()
+DebuggerScope* DebuggerCallFrame::scope(VM& vm)
 {
     ASSERT(isValid());
     if (!isValid())
         return nullptr;
 
     if (!m_scope) {
-        VM& vm = m_validMachineFrame->deprecatedVM();
         JSScope* scope;
-        CodeBlock* codeBlock = m_validMachineFrame->codeBlock();
+        CodeBlock* codeBlock = m_validMachineFrame->isNativeCalleeFrame() ? nullptr : m_validMachineFrame->codeBlock();
         if (isTailDeleted())
             scope = m_shadowChickenFrame.scope;
         else if (codeBlock && codeBlock->scopeRegister().isValid())
             scope = m_validMachineFrame->scope(codeBlock->scopeRegister().offset());
-        else if (JSCallee* callee = jsDynamicCast<JSCallee*>(vm, m_validMachineFrame->jsCallee()))
+        else if (JSCallee* callee = jsDynamicCast<JSCallee*>(m_validMachineFrame->jsCallee()))
             scope = callee->scope();
         else
             scope = m_validMachineFrame->lexicalGlobalObject(vm)->globalLexicalEnvironment();
@@ -175,7 +164,7 @@ DebuggerScope* DebuggerCallFrame::scope()
     return m_scope.get();
 }
 
-DebuggerCallFrame::Type DebuggerCallFrame::type() const
+DebuggerCallFrame::Type DebuggerCallFrame::type(VM&) const
 {
     ASSERT(isValid());
     if (!isValid())
@@ -184,7 +173,7 @@ DebuggerCallFrame::Type DebuggerCallFrame::type() const
     if (isTailDeleted())
         return FunctionType;
 
-    if (jsDynamicCast<JSFunction*>(m_validMachineFrame->deprecatedVM(), m_validMachineFrame->jsCallee()))
+    if (jsDynamicCast<JSFunction*>(m_validMachineFrame->jsCallee()))
         return FunctionType;
 
     return ProgramType;
@@ -203,7 +192,7 @@ JSValue DebuggerCallFrame::thisValue(VM& vm) const
         codeBlock = m_shadowChickenFrame.codeBlock;
     } else {
         thisValue = m_validMachineFrame->thisValue();
-        codeBlock = m_validMachineFrame->codeBlock();
+        codeBlock = m_validMachineFrame->isNativeCalleeFrame() ? nullptr : m_validMachineFrame->codeBlock();
     }
 
     if (!thisValue)
@@ -216,7 +205,7 @@ JSValue DebuggerCallFrame::thisValue(VM& vm) const
 }
 
 // Evaluate some JavaScript code in the scope of this frame.
-JSValue DebuggerCallFrame::evaluateWithScopeExtension(const String& script, JSObject* scopeExtensionObject, NakedPtr<Exception>& exception)
+JSValue DebuggerCallFrame::evaluateWithScopeExtension(VM& vm, const String& script, JSObject* scopeExtensionObject, NakedPtr<Exception>& exception)
 {
     CallFrame* callFrame = nullptr;
     CodeBlock* codeBlock = nullptr;
@@ -230,7 +219,7 @@ JSValue DebuggerCallFrame::evaluateWithScopeExtension(const String& script, JSOb
             if (debuggerCallFrame->isTailDeleted())
                 codeBlock = debuggerCallFrame->m_shadowChickenFrame.codeBlock;
             else
-                codeBlock = callFrame->codeBlock();
+                codeBlock = callFrame->isNativeCalleeFrame() ? nullptr : callFrame->codeBlock();
         }
 
         if (callFrame && codeBlock)
@@ -242,7 +231,6 @@ JSValue DebuggerCallFrame::evaluateWithScopeExtension(const String& script, JSOb
     if (!callFrame || !codeBlock)
         return jsUndefined();
 
-    VM& vm = callFrame->deprecatedVM();
     JSLockHolder lock(vm);
     auto catchScope = DECLARE_CATCH_SCOPE(vm);
     
@@ -260,11 +248,10 @@ JSValue DebuggerCallFrame::evaluateWithScopeExtension(const String& script, JSOb
 
     TDZEnvironment variablesUnderTDZ;
     PrivateNameEnvironment privateNameEnvironment;
-    JSScope::collectClosureVariablesUnderTDZ(scope()->jsScope(), variablesUnderTDZ, privateNameEnvironment);
+    JSScope::collectClosureVariablesUnderTDZ(scope(vm)->jsScope(), variablesUnderTDZ, privateNameEnvironment);
 
-    ECMAMode ecmaMode = codeBlock->ownerExecutable()->isInStrictContext() ? ECMAMode::strict() : ECMAMode::sloppy();
-    auto* eval = DirectEvalExecutable::create(globalObject, makeSource(script, callFrame->callerSourceOrigin(vm)), codeBlock->unlinkedCodeBlock()->derivedContextType(), codeBlock->unlinkedCodeBlock()->needsClassFieldInitializer(), codeBlock->unlinkedCodeBlock()->privateBrandRequirement(), codeBlock->unlinkedCodeBlock()->isArrowFunction(), codeBlock->ownerExecutable()->isInsideOrdinaryFunction(), evalContextType, &variablesUnderTDZ, &privateNameEnvironment, ecmaMode);
-    if (UNLIKELY(catchScope.exception())) {
+    auto* eval = DirectEvalExecutable::create(globalObject, makeSource(script, callFrame->callerSourceOrigin(vm), SourceTaintedOrigin::Untainted), codeBlock->ownerExecutable()->lexicallyScopedFeatures(), codeBlock->unlinkedCodeBlock()->derivedContextType(), codeBlock->unlinkedCodeBlock()->needsClassFieldInitializer(), codeBlock->unlinkedCodeBlock()->privateBrandRequirement(), codeBlock->unlinkedCodeBlock()->isArrowFunction(), codeBlock->ownerExecutable()->isInsideOrdinaryFunction(), evalContextType, &variablesUnderTDZ, &privateNameEnvironment);
+    if (catchScope.exception()) [[unlikely]] {
         exception = catchScope.exception();
         catchScope.clearException();
         return jsUndefined();
@@ -273,10 +260,19 @@ JSValue DebuggerCallFrame::evaluateWithScopeExtension(const String& script, JSOb
     if (scopeExtensionObject) {
         JSScope* ignoredPreviousScope = globalObject->globalScope();
         globalObject->setGlobalScopeExtension(JSWithScope::create(vm, globalObject, ignoredPreviousScope, scopeExtensionObject));
+        eval->setTaintedByWithScope();
     }
 
-    JSValue result = vm.interpreter->execute(eval, globalObject, debuggerCallFrame->thisValue(vm), debuggerCallFrame->scope()->jsScope());
-    if (UNLIKELY(catchScope.exception())) {
+    JSValue result;
+    {
+        auto* jsScope = debuggerCallFrame->scope(vm)->jsScope();
+        VMEntryScope entryScope(vm, jsScope->globalObject());
+        if (vm.disallowVMEntryCount) [[unlikely]]
+            result = Interpreter::checkVMEntryPermission();
+        else
+            result = vm.interpreter.executeEval(eval, debuggerCallFrame->thisValue(vm), jsScope);
+    }
+    if (catchScope.exception()) [[unlikely]] {
         exception = catchScope.exception();
         catchScope.clearException();
     }
@@ -309,8 +305,9 @@ TextPosition DebuggerCallFrame::currentPosition(VM& vm)
     if (isTailDeleted()) {
         CodeBlock* codeBlock = m_shadowChickenFrame.codeBlock;
         if (std::optional<BytecodeIndex> bytecodeIndex = codeBlock->bytecodeIndexFromCallSiteIndex(m_shadowChickenFrame.callSiteIndex)) {
-            return TextPosition(OrdinalNumber::fromOneBasedInt(codeBlock->lineNumberForBytecodeIndex(*bytecodeIndex)),
-                OrdinalNumber::fromOneBasedInt(codeBlock->columnNumberForBytecodeIndex(*bytecodeIndex)));
+            auto lineColumn = codeBlock->lineColumnForBytecodeIndex(*bytecodeIndex);
+            return TextPosition(OrdinalNumber::fromOneBasedInt(lineColumn.line),
+                OrdinalNumber::fromOneBasedInt(lineColumn.column));
         }
     }
 
@@ -320,6 +317,8 @@ TextPosition DebuggerCallFrame::currentPosition(VM& vm)
 TextPosition DebuggerCallFrame::positionForCallFrame(VM& vm, CallFrame* callFrame)
 {
     LineAndColumnFunctor functor;
+    if (!callFrame)
+        return TextPosition(OrdinalNumber::fromOneBasedInt(0), OrdinalNumber::fromOneBasedInt(0));
     StackVisitor::visit(callFrame, vm, functor);
     return TextPosition(OrdinalNumber::fromOneBasedInt(functor.line()), OrdinalNumber::fromOneBasedInt(functor.column()));
 }
@@ -328,8 +327,10 @@ SourceID DebuggerCallFrame::sourceIDForCallFrame(CallFrame* callFrame)
 {
     if (!callFrame)
         return noSourceID;
+    if (callFrame->isNativeCalleeFrame())
+        return noSourceID;
     CodeBlock* codeBlock = callFrame->codeBlock();
-    if (!codeBlock || callFrame->callee().isWasm())
+    if (!codeBlock)
         return noSourceID;
     return codeBlock->ownerExecutable()->sourceID();
 }
