@@ -267,6 +267,8 @@ class Label
             }
             result = Label.forName(codeOrigin, name, @definedInFile)
             result.setGlobal() if global?
+            result.setUnalignedGlobal() unless aligned?
+            result.setAligned(@alignTo) if aligned? and @alignTo
             result.clearExtern unless extern?
             result
         else
@@ -286,6 +288,8 @@ class Label
             }
             result = Label.forName(codeOrigin, name, @definedInFile)
             result.setGlobal() if global?
+            result.setUnalignedGlobal() unless aligned?
+            result.setAligned(@alignTo) if aligned? and @alignTo
             result.clearExtern unless extern?
             result
         else
@@ -396,7 +400,7 @@ class Sequence
         @list.each {
             | item |
             if item.is_a? LocalLabel
-                mapping[item] = LocalLabel.unique(if comment then comment + "_" else "" end + item.cleanName)
+                mapping[item] = LocalLabel.unique(codeOrigin, if comment then comment + "_" else "" end + item.cleanName)
             end
         }
         
@@ -677,6 +681,11 @@ class FPRegisterID
     end
 end
 
+class VecRegisterID
+    def validate
+    end
+end
+
 class Address
     def validate
         validateChildren
