@@ -24,12 +24,14 @@ class TSymbolTable;
 // Replace all gl_FragColor with gl_FragData[0], and in the end of main() function,
 // assign gl_FragData[1] ... gl_FragData[maxDrawBuffers - 1] with gl_FragData[0].
 // If gl_FragColor is in outputVariables, it is replaced by gl_FragData.
-ANGLE_NO_DISCARD bool EmulateGLFragColorBroadcast(TCompiler *compiler,
-                                                  TIntermBlock *root,
-                                                  int maxDrawBuffers,
-                                                  std::vector<ShaderVariable> *outputVariables,
-                                                  TSymbolTable *symbolTable,
-                                                  int shaderVersion);
+// Similarly replace all gl_SecondaryFragColorEXT with gl_SecondaryFragDataEXT[0].
+[[nodiscard]] bool EmulateGLFragColorBroadcast(TCompiler *compiler,
+                                               TIntermBlock *root,
+                                               int maxDrawBuffers,
+                                               int maxDualSourceDrawBuffers,
+                                               std::vector<ShaderVariable> *outputVariables,
+                                               TSymbolTable *symbolTable,
+                                               int shaderVersion);
 }  // namespace sh
 
 #endif  // COMPILER_TRANSLATOR_TREEOPS_EMULATEGLFRAGCOLORBROADCAST_H_
