@@ -32,6 +32,7 @@ class IOSurfaceSurfaceCGL : public SurfaceGL
 {
   public:
     IOSurfaceSurfaceCGL(const egl::SurfaceState &state,
+                        RendererGL *renderer,
                         CGLContextObj cglContext,
                         EGLClientBuffer buffer,
                         const egl::AttributeMap &attribs);
@@ -41,7 +42,7 @@ class IOSurfaceSurfaceCGL : public SurfaceGL
     egl::Error makeCurrent(const gl::Context *context) override;
     egl::Error unMakeCurrent(const gl::Context *context) override;
 
-    egl::Error swap(const gl::Context *context) override;
+    egl::Error swap(const gl::Context *context, SurfaceSwapFeedback *feedback) override;
     egl::Error postSubBuffer(const gl::Context *context,
                              EGLint x,
                              EGLint y,
@@ -52,7 +53,7 @@ class IOSurfaceSurfaceCGL : public SurfaceGL
                             gl::Texture *texture,
                             EGLint buffer) override;
     egl::Error releaseTexImage(const gl::Context *context, EGLint buffer) override;
-    void setSwapInterval(EGLint interval) override;
+    void setSwapInterval(const egl::Display *display, EGLint interval) override;
 
     EGLint getWidth() const override;
     EGLint getHeight() const override;
@@ -61,14 +62,22 @@ class IOSurfaceSurfaceCGL : public SurfaceGL
     EGLint getSwapBehavior() const override;
 
     static bool validateAttributes(EGLClientBuffer buffer, const egl::AttributeMap &attribs);
-    FramebufferImpl *createDefaultFramebuffer(const gl::Context *context,
-                                              const gl::FramebufferState &state) override;
 
     bool hasEmulatedAlphaChannel() const override;
 
+    egl::Error attachToFramebuffer(const gl::Context *context,
+                                   gl::Framebuffer *framebuffer) override;
+    egl::Error detachFromFramebuffer(const gl::Context *context,
+                                     gl::Framebuffer *framebuffer) override;
+
   private:
     angle::Result initializeAlphaChannel(const gl::Context *context, GLuint texture);
 
+    // TODO(geofflang): Don't store these, they are potentially specific to a single GL context.
+    // http://anglebug.com/40096492
+    const FunctionsGL *mFunctions;
+    StateManagerGL *mStateManager;
+
     CGLContextObj mCGLContext;
     IOSurfaceRef mIOSurface;
     int mWidth;
@@ -77,6 +86,9 @@ class IOSurfaceSurfaceCGL : public SurfaceGL
     int mFormatIndex;
 
     bool mAlphaInitialized;
+
+    GLuint mTextureID;
+    GLuint mFramebufferID;
 };
 
 }  // namespace rx
