@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010 Google Inc. All rights reserved.
  * Copyright (C) 2014 University of Washington. All rights reserved.
- * Copyright (C) 2017-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -33,6 +33,12 @@
 #include "config.h"
 #include <wtf/JSONValues.h>
 
+#include <functional>
+#include <wtf/ASCIICType.h>
+#include <wtf/CommaPrinter.h>
+#include <wtf/ZippedRange.h>
+#include <wtf/text/MakeString.h>
+#include <wtf/text/ParsingUtilities.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace WTF {
@@ -57,127 +63,121 @@ enum class Token {
     Invalid,
 };
 
-const char* const nullToken = "null";
-const char* const trueToken = "true";
-const char* const falseToken = "false";
+constexpr auto nullToken = "null"_s;
+constexpr auto trueToken = "true"_s;
+constexpr auto falseToken = "false"_s;
 
 template<typename CodeUnit>
-bool parseConstToken(const CodeUnit* start, const CodeUnit* end, const CodeUnit** tokenEnd, const char* token)
+bool parseConstToken(std::span<const CodeUnit> data, std::span<const CodeUnit>& tokenEnd, ASCIILiteral token)
 {
-    while (start < end && *token != '\0' && *start++ == *token++) { }
-
-    if (*token != '\0')
+    if (data.size() < token.length())
         return false;
 
-    *tokenEnd = start;
+    for (auto [character, tokenCharacter] : zippedRange(data, token.span8())) {
+        if (character != tokenCharacter)
+            return false;
+    }
+
+    tokenEnd = data.subspan(token.length());
     return true;
 }
 
 template<typename CodeUnit>
-bool readInt(const CodeUnit* start, const CodeUnit* end, const CodeUnit** tokenEnd, bool canHaveLeadingZeros)
+bool readInt(std::span<const CodeUnit> data, std::span<const CodeUnit>& tokenEnd, bool canHaveLeadingZeros)
 {
-    if (start == end)
+    if (data.empty())
         return false;
 
-    bool haveLeadingZero = '0' == *start;
-    int length = 0;
-    while (start < end && '0' <= *start && *start <= '9') {
-        ++start;
-        ++length;
-    }
+    bool hasLeadingZero = data[0] == '0';
+    size_t originalSize = data.size();
+    skipWhile<isASCIIDigit>(data);
 
+    size_t length = originalSize - data.size();
     if (!length)
         return false;
 
-    if (!canHaveLeadingZeros && length > 1 && haveLeadingZero)
+    if (!canHaveLeadingZeros && length > 1 && hasLeadingZero)
         return false;
 
-    *tokenEnd = start;
+    tokenEnd = data;
     return true;
 }
 
 template<typename CodeUnit>
-bool parseNumberToken(const CodeUnit* start, const CodeUnit* end, const CodeUnit** tokenEnd)
+bool parseNumberToken(std::span<const CodeUnit> data, std::span<const CodeUnit>& tokenEnd)
 {
     // We just grab the number here. We validate the size in DecodeNumber.
     // According to RFC 4627, a valid number is: [minus] int [frac] [exp]
-    if (start == end)
+    if (data.empty())
         return false;
 
-    CodeUnit c = *start;
-    if ('-' == c)
-        ++start;
+    skipExactly(data, '-');
 
-    if (!readInt(start, end, &start, false))
+    if (!readInt(data, data, false))
         return false;
 
-    if (start == end) {
-        *tokenEnd = start;
+    if (data.empty()) {
+        tokenEnd = data;
         return true;
     }
 
     // Optional fraction part.
-    c = *start;
-    if ('.' == c) {
-        ++start;
-        if (!readInt(start, end, &start, true))
+    if (skipExactly(data, '.')) {
+        if (!readInt(data, data, true))
             return false;
-        if (start == end) {
-            *tokenEnd = start;
+        if (data.empty()) {
+            tokenEnd = data;
             return true;
         }
-        c = *start;
     }
 
     // Optional exponent part.
-    if ('e' == c || 'E' == c) {
-        ++start;
-        if (start == end)
+    if (skipExactly(data, 'e') || skipExactly(data, 'E')) {
+        if (data.empty())
             return false;
-        c = *start;
-        if ('-' == c || '+' == c) {
-            ++start;
-            if (start == end)
+        if (skipExactly(data, '-') || skipExactly(data, '+')) {
+            if (data.empty())
                 return false;
         }
-        if (!readInt(start, end, &start, true))
+        if (!readInt(data, data, true))
             return false;
     }
 
-    *tokenEnd = start;
+    tokenEnd = data;
     return true;
 }
 
 template<typename CodeUnit>
-bool readHexDigits(const CodeUnit* start, const CodeUnit* end, const CodeUnit** tokenEnd, int digits)
+bool readHexDigits(std::span<const CodeUnit> data, std::span<const CodeUnit>& tokenEnd, unsigned digits)
 {
-    if (end - start < digits)
+    if (data.size() < digits)
         return false;
 
-    for (int i = 0; i < digits; ++i) {
-        if (!isASCIIHexDigit(*start++))
+    for (unsigned i = 0; i < digits; ++i) {
+        if (!isASCIIHexDigit(data.front()))
             return false;
+        skip(data, 1);
     }
 
-    *tokenEnd = start;
+    tokenEnd = data;
     return true;
 }
 
 template<typename CodeUnit>
-bool parseStringToken(const CodeUnit* start, const CodeUnit* end, const CodeUnit** tokenEnd)
+bool parseStringToken(std::span<const CodeUnit> data, std::span<const CodeUnit>& tokenEnd)
 {
-    while (start < end) {
-        CodeUnit c = *start++;
-        if ('\\' == c && start < end) {
-            c = *start++;
+    while (!data.empty()) {
+        CodeUnit c = consume(data);
+        if ('\\' == c && !data.empty()) {
+            c = consume(data);
             // Make sure the escaped char is valid.
             switch (c) {
             case 'x':
-                if (!readHexDigits(start, end, &start, 2))
+                if (!readHexDigits(data, data, 2))
                     return false;
                 break;
             case 'u':
-                if (!readHexDigits(start, end, &start, 4))
+                if (!readHexDigits(data, data, 4))
                     return false;
                 break;
             case '\\':
@@ -194,7 +194,7 @@ bool parseStringToken(const CodeUnit* start, const CodeUnit* end, const CodeUnit
                 return false;
             }
         } else if ('"' == c) {
-            *tokenEnd = start;
+            tokenEnd = data;
             return true;
         }
     }
@@ -203,46 +203,45 @@ bool parseStringToken(const CodeUnit* start, const CodeUnit* end, const CodeUnit
 }
 
 template<typename CodeUnit>
-Token parseToken(const CodeUnit* start, const CodeUnit* end, const CodeUnit** tokenStart, const CodeUnit** tokenEnd)
+Token parseToken(std::span<const CodeUnit> data, std::span<const CodeUnit>& tokenStart, std::span<const CodeUnit>& tokenEnd)
 {
-    while (start < end && isSpaceOrNewline(*start))
-        ++start;
+    skipWhile<isASCIIWhitespaceWithoutFF>(data);
 
-    if (start == end)
+    if (data.empty())
         return Token::Invalid;
 
-    *tokenStart = start;
+    tokenStart = data;
 
-    switch (*start) {
+    switch (data.front()) {
     case 'n':
-        if (parseConstToken(start, end, tokenEnd, nullToken))
+        if (parseConstToken(data, tokenEnd, nullToken))
             return Token::Null;
         break;
     case 't':
-        if (parseConstToken(start, end, tokenEnd, trueToken))
+        if (parseConstToken(data, tokenEnd, trueToken))
             return Token::BoolTrue;
         break;
     case 'f':
-        if (parseConstToken(start, end, tokenEnd, falseToken))
+        if (parseConstToken(data, tokenEnd, falseToken))
             return Token::BoolFalse;
         break;
     case '[':
-        *tokenEnd = start + 1;
+        tokenEnd = data.subspan(1);
         return Token::ArrayBegin;
     case ']':
-        *tokenEnd = start + 1;
+        tokenEnd = data.subspan(1);
         return Token::ArrayEnd;
     case ',':
-        *tokenEnd = start + 1;
+        tokenEnd = data.subspan(1);
         return Token::ListSeparator;
     case '{':
-        *tokenEnd = start + 1;
+        tokenEnd = data.subspan(1);
         return Token::ObjectBegin;
     case '}':
-        *tokenEnd = start + 1;
+        tokenEnd = data.subspan(1);
         return Token::ObjectEnd;
     case ':':
-        *tokenEnd = start + 1;
+        tokenEnd = data.subspan(1);
         return Token::ObjectPairSeparator;
     case '0':
     case '1':
@@ -255,11 +254,11 @@ Token parseToken(const CodeUnit* start, const CodeUnit* end, const CodeUnit** to
     case '8':
     case '9':
     case '-':
-        if (parseNumberToken(start, end, tokenEnd))
+        if (parseNumberToken(data, tokenEnd))
             return Token::Number;
         break;
     case '"':
-        if (parseStringToken(start + 1, end, tokenEnd))
+        if (parseStringToken(data.subspan(1), tokenEnd))
             return Token::String;
         break;
     }
@@ -268,17 +267,17 @@ Token parseToken(const CodeUnit* start, const CodeUnit* end, const CodeUnit** to
 }
 
 template<typename CodeUnit>
-bool decodeString(const CodeUnit* start, const CodeUnit* end, StringBuilder& output)
+bool decodeString(std::span<const CodeUnit> data, StringBuilder& output)
 {
-    while (start < end) {
-        UChar c = *start++;
+    while (!data.empty()) {
+        char16_t c = consume(data);
         if ('\\' != c) {
             output.append(c);
             continue;
         }
-        if (UNLIKELY(start >= end))
+        if (data.empty()) [[unlikely]]
             return false;
-        c = *start++;
+        c = consume(data);
         switch (c) {
         case '"':
         case '/':
@@ -303,16 +302,16 @@ bool decodeString(const CodeUnit* start, const CodeUnit* end, StringBuilder& out
             c = '\v';
             break;
         case 'x':
-            if (UNLIKELY(start + 1 >= end))
+            if (data.size() < 2) [[unlikely]]
                 return false;
-            c = toASCIIHexValue(start[0], start[1]);
-            start += 2;
+            c = toASCIIHexValue(data[0], data[1]);
+            skip(data, 2);
             break;
         case 'u':
-            if (UNLIKELY(start + 3 >= end))
+            if (data.size() < 4) [[unlikely]]
                 return false;
-            c = toASCIIHexValue(start[0], start[1]) << 8 | toASCIIHexValue(start[2], start[3]);
-            start += 4;
+            c = toASCIIHexValue(data[0], data[1]) << 8 | toASCIIHexValue(data[2], data[3]);
+            skip(data, 4);
             break;
         default:
             return false;
@@ -324,19 +323,16 @@ bool decodeString(const CodeUnit* start, const CodeUnit* end, StringBuilder& out
 }
 
 template<typename CodeUnit>
-bool decodeString(const CodeUnit* start, const CodeUnit* end, String& output)
+bool decodeString(std::span<const CodeUnit> data, String& output)
 {
-    if (start == end) {
+    if (data.empty()) {
         output = emptyString();
         return true;
     }
 
-    if (start > end)
-        return false;
-
     StringBuilder buffer;
-    buffer.reserveCapacity(end - start);
-    if (!decodeString(start, end, buffer))
+    buffer.reserveCapacity(data.size());
+    if (!decodeString(data, buffer))
         return false;
 
     output = buffer.toString();
@@ -344,15 +340,15 @@ bool decodeString(const CodeUnit* start, const CodeUnit* end, String& output)
 }
 
 template<typename CodeUnit>
-RefPtr<JSON::Value> buildValue(const CodeUnit* start, const CodeUnit* end, const CodeUnit** valueTokenEnd, int depth)
+RefPtr<JSON::Value> buildValue(std::span<const CodeUnit> data, std::span<const CodeUnit>& valueTokenEnd, int depth)
 {
     if (depth > stackLimit)
         return nullptr;
 
     RefPtr<JSON::Value> result;
-    const CodeUnit* tokenStart;
-    const CodeUnit* tokenEnd;
-    Token token = parseToken(start, end, &tokenStart, &tokenEnd);
+    std::span<const CodeUnit> tokenStart;
+    std::span<const CodeUnit> tokenEnd;
+    Token token = parseToken(data, tokenStart, tokenEnd);
     switch (token) {
     case Token::Invalid:
         return nullptr;
@@ -367,7 +363,7 @@ RefPtr<JSON::Value> buildValue(const CodeUnit* start, const CodeUnit* end, const
         break;
     case Token::Number: {
         bool ok;
-        double value = charactersToDouble(tokenStart, tokenEnd - tokenStart, &ok);
+        double value = charactersToDouble(tokenStart.first(tokenEnd.data() - tokenStart.data()), &ok);
         if (!ok)
             return nullptr;
         result = JSON::Value::create(value);
@@ -375,7 +371,9 @@ RefPtr<JSON::Value> buildValue(const CodeUnit* start, const CodeUnit* end, const
     }
     case Token::String: {
         String value;
-        bool ok = decodeString(tokenStart + 1, tokenEnd - 1, value);
+        if (tokenEnd.data() - tokenStart.data() < 2)
+            return nullptr;
+        bool ok = decodeString(tokenStart.subspan(1, std::to_address(tokenEnd.begin()) - std::to_address(tokenStart.begin()) - 2), value);
         if (!ok)
             return nullptr;
         result = JSON::Value::create(value);
@@ -383,20 +381,20 @@ RefPtr<JSON::Value> buildValue(const CodeUnit* start, const CodeUnit* end, const
     }
     case Token::ArrayBegin: {
         Ref<JSON::Array> array = JSON::Array::create();
-        start = tokenEnd;
-        token = parseToken(start, end, &tokenStart, &tokenEnd);
+        data = tokenEnd;
+        token = parseToken(data, tokenStart, tokenEnd);
         while (token != Token::ArrayEnd) {
-            RefPtr<JSON::Value> arrayNode = buildValue(start, end, &tokenEnd, depth + 1);
+            RefPtr<JSON::Value> arrayNode = buildValue(data, tokenEnd, depth + 1);
             if (!arrayNode)
                 return nullptr;
             array->pushValue(arrayNode.releaseNonNull());
 
             // After a list value, we expect a comma or the end of the list.
-            start = tokenEnd;
-            token = parseToken(start, end, &tokenStart, &tokenEnd);
+            data = tokenEnd;
+            token = parseToken(data, tokenStart, tokenEnd);
             if (token == Token::ListSeparator) {
-                start = tokenEnd;
-                token = parseToken(start, end, &tokenStart, &tokenEnd);
+                data = tokenEnd;
+                token = parseToken(data, tokenStart, tokenEnd);
                 if (token == Token::ArrayEnd)
                     return nullptr;
             } else if (token != Token::ArrayEnd) {
@@ -404,40 +402,41 @@ RefPtr<JSON::Value> buildValue(const CodeUnit* start, const CodeUnit* end, const
                 return nullptr;
             }
         }
-        if (token != Token::ArrayEnd)
-            return nullptr;
+        ASSERT(token == Token::ArrayEnd);
         result = WTFMove(array);
         break;
     }
     case Token::ObjectBegin: {
         Ref<JSON::Object> object = JSON::Object::create();
-        start = tokenEnd;
-        token = parseToken(start, end, &tokenStart, &tokenEnd);
+        data = tokenEnd;
+        token = parseToken(data, tokenStart, tokenEnd);
         while (token != Token::ObjectEnd) {
             if (token != Token::String)
                 return nullptr;
             String key;
-            if (!decodeString(tokenStart + 1, tokenEnd - 1, key))
+            if (tokenEnd.data() - tokenStart.data() < 2)
                 return nullptr;
-            start = tokenEnd;
+            if (!decodeString(tokenStart.subspan(1, std::to_address(tokenEnd.begin()) - std::to_address(tokenStart.begin()) - 2), key))
+                return nullptr;
+            data = tokenEnd;
 
-            token = parseToken(start, end, &tokenStart, &tokenEnd);
+            token = parseToken(data, tokenStart, tokenEnd);
             if (token != Token::ObjectPairSeparator)
                 return nullptr;
-            start = tokenEnd;
+            data = tokenEnd;
 
-            RefPtr<JSON::Value> value = buildValue(start, end, &tokenEnd, depth + 1);
+            RefPtr<JSON::Value> value = buildValue(data, tokenEnd, depth + 1);
             if (!value)
                 return nullptr;
             object->setValue(key, value.releaseNonNull());
-            start = tokenEnd;
+            data = tokenEnd;
 
             // After a key/value pair, we expect a comma or the end of the
             // object.
-            token = parseToken(start, end, &tokenStart, &tokenEnd);
+            token = parseToken(data, tokenStart, tokenEnd);
             if (token == Token::ListSeparator) {
-                start = tokenEnd;
-                token = parseToken(start, end, &tokenStart, &tokenEnd);
+                data = tokenEnd;
+                token = parseToken(data, tokenStart, tokenEnd);
                 if (token == Token::ObjectEnd)
                     return nullptr;
             } else if (token != Token::ObjectEnd) {
@@ -445,8 +444,7 @@ RefPtr<JSON::Value> buildValue(const CodeUnit* start, const CodeUnit* end, const
                 return nullptr;
             }
         }
-        if (token != Token::ObjectEnd)
-            return nullptr;
+        ASSERT(token == Token::ObjectEnd);
         result = WTFMove(object);
         break;
     }
@@ -455,18 +453,43 @@ RefPtr<JSON::Value> buildValue(const CodeUnit* start, const CodeUnit* end, const
         // We got a token that's not a value.
         return nullptr;
     }
-    *valueTokenEnd = tokenEnd;
+    valueTokenEnd = tokenEnd;
     return result;
 }
 
-inline void appendDoubleQuotedString(StringBuilder& builder, StringView string)
+} // anonymous namespace
+
+template<typename Visitor> constexpr decltype(auto) Value::visitDerived(Visitor&& visitor)
 {
-    builder.append('"');
-    Value::escapeString(builder, string);
-    builder.append('"');
+    switch (m_type) {
+    case Type::Null:
+    case Type::Boolean:
+    case Type::Double:
+    case Type::Integer:
+    case Type::String:
+        return std::invoke(std::forward<Visitor>(visitor), static_cast<Value&>(*this));
+    case Type::Object:
+        return std::invoke(std::forward<Visitor>(visitor), static_cast<Object&>(*this));
+    case Type::Array:
+        return std::invoke(std::forward<Visitor>(visitor), static_cast<Array&>(*this));
+    }
+    RELEASE_ASSERT_NOT_REACHED();
 }
 
-} // anonymous namespace
+template<typename Visitor> constexpr decltype(auto) Value::visitDerived(Visitor&& visitor) const
+{
+    return const_cast<Value&>(*this).visitDerived([&](auto& derived) {
+        return std::invoke(std::forward<Visitor>(visitor), std::as_const(derived));
+    });
+}
+
+void Value::operator delete(Value* value, std::destroying_delete_t)
+{
+    value->visitDerived([](auto& derived) {
+        std::destroy_at(&derived);
+        std::decay_t<decltype(derived)>::freeAfterDestruction(&derived);
+    });
+}
 
 Ref<Value> Value::null()
 {
@@ -493,33 +516,13 @@ Ref<Value> Value::create(const String& value)
     return adoptRef(*new Value(value));
 }
 
-RefPtr<Value> Value::asValue()
+RefPtr<Value> Value::parseJSON(StringView json)
 {
-    return this;
-}
-
-RefPtr<Object> Value::asObject()
-{
-    return nullptr;
-}
-
-RefPtr<const Object> Value::asObject() const
-{
-    return nullptr;
-}
-
-RefPtr<Array> Value::asArray()
-{
-    return nullptr;
-}
-
-RefPtr<Value> Value::parseJSON(const String& json)
-{
-    auto containsNonSpace = [] (const auto* begin, const auto* end) {
-        if (!begin)
+    auto containsNonSpace = [] (auto span) {
+        if (!span.data())
             return false;
-        for (const auto* it = begin; it < end; it++) {
-            if (!isSpaceOrNewline(*it))
+        for (auto character : span) {
+            if (!isASCIIWhitespaceWithoutFF(character))
                 return true;
         }
         return false;
@@ -527,61 +530,27 @@ RefPtr<Value> Value::parseJSON(const String& json)
 
     RefPtr<Value> result;
     if (json.is8Bit()) {
-        const LChar* start = json.characters8();
-        const LChar* end = start + json.length();
-        const LChar* tokenEnd { nullptr };
-        result = buildValue(start, end, &tokenEnd, 0);
-        if (containsNonSpace(tokenEnd, end))
+        auto data = json.span8();
+        std::span<const LChar> tokenEnd;
+        result = buildValue(data, tokenEnd, 0);
+        if (containsNonSpace(tokenEnd))
             return nullptr;
     } else {
-        const UChar* start = json.characters16();
-        const UChar* end = start + json.length();
-        const UChar* tokenEnd { nullptr };
-        result = buildValue(start, end, &tokenEnd, 0);
-        if (containsNonSpace(tokenEnd, end))
+        auto data = json.span16();
+        std::span<const char16_t> tokenEnd;
+        result = buildValue(data, tokenEnd, 0);
+        if (containsNonSpace(tokenEnd))
             return nullptr;
     }
     return result;
 }
 
-void Value::escapeString(StringBuilder& builder, StringView string)
+std::optional<Ref<Value>> Value::optionalParseJSON(StringView json)
 {
-    for (UChar codeUnit : string.codeUnits()) {
-        switch (codeUnit) {
-        case '\b':
-            builder.append("\\b");
-            continue;
-        case '\f':
-            builder.append("\\f");
-            continue;
-        case '\n':
-            builder.append("\\n");
-            continue;
-        case '\r':
-            builder.append("\\r");
-            continue;
-        case '\t':
-            builder.append("\\t");
-            continue;
-        case '\\':
-            builder.append("\\\\");
-            continue;
-        case '"':
-            builder.append("\\\"");
-            continue;
-        }
-        // We escape < and > to prevent script execution.
-        if (codeUnit >= 32 && codeUnit < 127 && codeUnit != '<' && codeUnit != '>') {
-            builder.append(codeUnit);
-            continue;
-        }
-        // We could encode characters >= 127 as UTF-8 instead of \u escape sequences.
-        // We could handle surrogates here if callers wanted that; for now we just
-        // write them out as a \u sequence, so a surrogate pair appears as two of them.
-        builder.append("\\u",
-            upperNibbleToASCIIHexDigit(codeUnit >> 8), lowerNibbleToASCIIHexDigit(codeUnit >> 8),
-            upperNibbleToASCIIHexDigit(codeUnit), lowerNibbleToASCIIHexDigit(codeUnit));
-    }
+    auto value = parseJSON(json);
+    if (value)
+        return value.releaseNonNull();
+    return std::nullopt;
 }
 
 String Value::toJSONString() const
@@ -620,27 +589,50 @@ String Value::asString() const
     return m_value.string;
 }
 
-void Value::writeJSON(StringBuilder& output) const
+void Value::dump(PrintStream& out) const
 {
     switch (m_type) {
     case Type::Null:
-        output.append("null");
+        out.print("null"_s);
         break;
     case Type::Boolean:
-        if (m_value.boolean)
-            output.append("true");
-        else
-            output.append("false");
+        out.print(m_value.boolean ? "true"_s : "false"_s);
         break;
-    case Type::String:
-        appendDoubleQuotedString(output, m_value.string);
+    case Type::String: {
+        StringBuilder builder;
+        builder.appendQuotedJSONString(m_value.string);
+        out.print(builder.toString());
         break;
+    }
     case Type::Double:
     case Type::Integer: {
         if (!std::isfinite(m_value.number))
-            output.append("null");
+            out.print("null"_s);
         else
-            output.append(m_value.number);
+            out.print(makeString(m_value.number));
+        break;
+    }
+    case Type::Object: {
+        auto& object = *static_cast<const ObjectBase*>(this);
+        CommaPrinter comma(","_s);
+        out.print("{"_s);
+        for (const auto& key : object.m_order) {
+            auto findResult = object.m_map.find(key);
+            ASSERT(findResult != object.m_map.end());
+            StringBuilder builder;
+            builder.appendQuotedJSONString(findResult->key);
+            out.print(comma, builder.toString(), ":"_s, findResult->value.get());
+        }
+        out.print("}"_s);
+        break;
+    }
+    case Type::Array: {
+        auto& array = *static_cast<const ArrayBase*>(this);
+        CommaPrinter comma(","_s);
+        out.print("["_s);
+        for (auto& value : array.m_map)
+            out.print(comma, value.get());
+        out.print("]"_s);
         break;
     }
     default:
@@ -648,33 +640,62 @@ void Value::writeJSON(StringBuilder& output) const
     }
 }
 
+void Value::writeJSON(StringBuilder& output) const
+{
+    visitDerived([&](auto& derived) {
+        derived.writeJSONImpl(output);
+    });
+}
+
+void Value::writeJSONImpl(StringBuilder& output) const
+{
+    switch (m_type) {
+    case Type::Null:
+        output.append("null"_s);
+        break;
+    case Type::Boolean:
+        if (m_value.boolean)
+            output.append("true"_s);
+        else
+            output.append("false"_s);
+        break;
+    case Type::String:
+        output.appendQuotedJSONString(m_value.string);
+        break;
+    case Type::Double:
+    case Type::Integer: {
+        if (!std::isfinite(m_value.number))
+            output.append("null"_s);
+        else
+            output.append(m_value.number);
+        break;
+    }
+    case Type::Object:
+    case Type::Array:
+        ASSERT_NOT_REACHED();
+    }
+}
+
 size_t Value::memoryCost() const
 {
-    size_t memoryCost = sizeof(this);
+    return visitDerived([&](auto& derived) {
+        return derived.memoryCostImpl();
+    });
+}
+
+size_t Value::memoryCostImpl() const
+{
+    size_t memoryCost = sizeof(*this);
     if (m_type == Type::String && m_value.string)
         memoryCost += m_value.string->sizeInBytes();
     return memoryCost;
 }
 
-ObjectBase::~ObjectBase()
-{
-}
+ObjectBase::~ObjectBase() = default;
 
-RefPtr<Object> ObjectBase::asObject()
+size_t ObjectBase::memoryCostImpl() const
 {
-    static_assert(sizeof(Object) == sizeof(ObjectBase), "cannot cast");
-    return static_cast<Object*>(this);
-}
-
-RefPtr<const Object> ObjectBase::asObject() const
-{
-    static_assert(sizeof(Object) == sizeof(ObjectBase), "cannot cast");
-    return static_cast<const Object*>(this);
-}
-
-size_t ObjectBase::memoryCost() const
-{
-    size_t memoryCost = Value::memoryCost();
+    size_t memoryCost = sizeof(*this);
     for (const auto& entry : m_map)
         memoryCost += entry.key.sizeInBytes() + entry.value->memoryCost();
     return memoryCost;
@@ -742,7 +763,7 @@ void ObjectBase::remove(const String& name)
     m_order.removeFirst(name);
 }
 
-void ObjectBase::writeJSON(StringBuilder& output) const
+void ObjectBase::writeJSONImpl(StringBuilder& output) const
 {
     output.append('{');
     for (size_t i = 0; i < m_order.size(); ++i) {
@@ -750,7 +771,7 @@ void ObjectBase::writeJSON(StringBuilder& output) const
         ASSERT(findResult != m_map.end());
         if (i)
             output.append(',');
-        appendDoubleQuotedString(output, findResult->key);
+        output.appendQuotedJSONString(findResult->key);
         output.append(':');
         findResult->value->writeJSON(output);
     }
@@ -762,23 +783,18 @@ ObjectBase::ObjectBase()
 {
 }
 
-ArrayBase::~ArrayBase()
-{
-}
+ArrayBase::~ArrayBase() = default;
 
-RefPtr<Array> ArrayBase::asArray()
-{
-    static_assert(sizeof(ArrayBase) == sizeof(Array), "cannot cast");
-    return static_cast<Array*>(this);
-}
-
-void ArrayBase::writeJSON(StringBuilder& output) const
+void ArrayBase::writeJSONImpl(StringBuilder& output) const
 {
     output.append('[');
-    for (auto it = m_map.begin(); it != m_map.end(); ++it) {
-        if (it != m_map.begin())
+    bool isFirst = true;
+    for (auto& value : m_map) {
+        if (isFirst)
+            isFirst = false;
+        else
             output.append(',');
-        (*it)->writeJSON(output);
+        value->writeJSON(output);
     }
     output.append(']');
 }
@@ -804,9 +820,9 @@ Ref<Array> Array::create()
     return adoptRef(*new Array);
 }
 
-size_t ArrayBase::memoryCost() const
+size_t ArrayBase::memoryCostImpl() const
 {
-    size_t memoryCost = Value::memoryCost();
+    size_t memoryCost = sizeof(*this);
     for (const auto& item : m_map)
         memoryCost += item->memoryCost();
     return memoryCost;
