@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Google Inc.  All rights reserved.
+ * Copyright (C) 2011 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -33,20 +33,20 @@
 #include "ThreadableWebSocketChannel.h"
 #include "WebSocketChannelClient.h"
 #include "WorkerGlobalScope.h"
+#include "WorkerLoaderProxy.h"
 #include <wtf/RefCounted.h>
 #include <wtf/RefPtr.h>
+#include <wtf/TZoneMalloc.h>
 #include <wtf/text/WTFString.h>
 
 namespace WebCore {
 
 class ScriptExecutionContext;
 class ThreadableWebSocketChannelClientWrapper;
-class WorkerGlobalScope;
-class WorkerLoaderProxy;
 class WorkerRunLoop;
 
 class WorkerThreadableWebSocketChannel final : public RefCounted<WorkerThreadableWebSocketChannel>, public ThreadableWebSocketChannel {
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_TZONE_ALLOCATED_EXPORT(WorkerThreadableWebSocketChannel, WEBCORE_EXPORT);
 public:
     static Ref<ThreadableWebSocketChannel> create(WorkerGlobalScope& workerGlobalScope, WebSocketChannelClient& client, const String& taskMode, SocketProvider& provider)
     {
@@ -58,50 +58,52 @@ public:
     ConnectStatus connect(const URL&, const String& protocol) final;
     String subprotocol() final;
     String extensions() final;
-    ThreadableWebSocketChannel::SendResult send(const String& message) final;
-    ThreadableWebSocketChannel::SendResult send(const JSC::ArrayBuffer&, unsigned byteOffset, unsigned byteLength) final;
-    ThreadableWebSocketChannel::SendResult send(Blob&) final;
-    unsigned bufferedAmount() const final;
+    void send(CString&&) final;
+    void send(const JSC::ArrayBuffer&, unsigned byteOffset, unsigned byteLength) final;
+    void send(Blob&) final;
     void close(int code, const String& reason) final;
-    void fail(const String& reason) final;
+    void fail(String&& reason) final;
     void disconnect() final; // Will suppress didClose().
     void suspend() final;
     void resume() final;
 
-    // Generated by the bridge.  The Peer and its bridge should have identical
+    // Generated by the bridge. The Peer and its bridge should have identical
     // lifetimes.
     class Peer : public WebSocketChannelClient {
-        WTF_MAKE_NONCOPYABLE(Peer); WTF_MAKE_FAST_ALLOCATED;
+        WTF_MAKE_TZONE_ALLOCATED(Peer);
+        WTF_MAKE_NONCOPYABLE(Peer);
     public:
-        Peer(Ref<ThreadableWebSocketChannelClientWrapper>&&, WorkerLoaderProxy&, ScriptExecutionContext&, const String& taskMode, SocketProvider&);
+        static Ref<Peer> create(Ref<ThreadableWebSocketChannelClientWrapper>&&, ScriptExecutionContext&, ScriptExecutionContextIdentifier, const String& taskMode, SocketProvider&);
         ~Peer();
 
         ConnectStatus connect(const URL&, const String& protocol);
-        void send(const String& message);
+        void send(CString&&);
         void send(const JSC::ArrayBuffer&);
         void send(Blob&);
         void bufferedAmount();
         void close(int code, const String& reason);
-        void fail(const String& reason);
+        void fail(String&& reason);
         void disconnect();
         void suspend();
         void resume();
 
         // WebSocketChannelClient functions.
         void didConnect() final;
-        void didReceiveMessage(const String& message) final;
+        void didReceiveMessage(String&& message) final;
         void didReceiveBinaryData(Vector<uint8_t>&&) final;
         void didUpdateBufferedAmount(unsigned bufferedAmount) final;
         void didStartClosingHandshake() final;
         void didClose(unsigned unhandledBufferedAmount, ClosingHandshakeCompletionStatus, unsigned short code, const String& reason) final;
-        void didReceiveMessageError() final;
+        void didReceiveMessageError(String&& reason) final;
         void didUpgradeURL() final;
 
     private:
-        Ref<ThreadableWebSocketChannelClientWrapper> m_workerClientWrapper;
-        WorkerLoaderProxy& m_loaderProxy;
+        Peer(Ref<ThreadableWebSocketChannelClientWrapper>&&, ScriptExecutionContext&, ScriptExecutionContextIdentifier, const String& taskMode, SocketProvider&);
+
+        ThreadSafeWeakPtr<ThreadableWebSocketChannelClientWrapper> m_workerClientWrapper;
         RefPtr<ThreadableWebSocketChannel> m_mainWebSocketChannel;
         String m_taskMode;
+        ScriptExecutionContextIdentifier m_workerContextIdentifier;
     };
 
     using RefCounted<WorkerThreadableWebSocketChannel>::ref;
@@ -119,14 +121,13 @@ private:
             return adoptRef(*new Bridge(WTFMove(workerClientWrapper), WTFMove(workerGlobalScope), taskMode, WTFMove(provider)));
         }
         ~Bridge();
-        void initialize();
+        void initialize(WorkerGlobalScope&);
         void connect(const URL&, const String& protocol);
-        ThreadableWebSocketChannel::SendResult send(const String& message);
-        ThreadableWebSocketChannel::SendResult send(const JSC::ArrayBuffer&, unsigned byteOffset, unsigned byteLength);
-        ThreadableWebSocketChannel::SendResult send(Blob&);
-        unsigned bufferedAmount();
+        void send(CString&&);
+        void send(const JSC::ArrayBuffer&, unsigned byteOffset, unsigned byteLength);
+        void send(Blob&);
         void close(int code, const String& reason);
-        void fail(const String& reason);
+        void fail(String&& reason);
         void disconnect();
         void suspend();
         void resume();
@@ -140,20 +141,17 @@ private:
         static void setWebSocketChannel(ScriptExecutionContext*, Bridge* thisPtr, Peer*, Ref<ThreadableWebSocketChannelClientWrapper>&&);
 
         // Executed on the main thread to create a Peer for this bridge.
-        static void mainThreadInitialize(ScriptExecutionContext&, WorkerLoaderProxy&, Ref<ThreadableWebSocketChannelClientWrapper>&&, const String& taskMode, Ref<SocketProvider>&&);
+        static RefPtr<Peer> mainThreadInitialize(ScriptExecutionContext&, WorkerThread&, ScriptExecutionContextIdentifier, Ref<ThreadableWebSocketChannelClientWrapper>&&, const String& taskMode, Ref<SocketProvider>&&);
 
         // Executed on the worker context's thread.
         void clearClientWrapper();
 
-        void setMethodNotCompleted();
-        void waitForMethodCompletion();
-
-        Ref<ThreadableWebSocketChannelClientWrapper> m_workerClientWrapper;
+        const Ref<ThreadableWebSocketChannelClientWrapper> m_workerClientWrapper;
         RefPtr<WorkerGlobalScope> m_workerGlobalScope;
         WorkerLoaderProxy& m_loaderProxy;
         String m_taskMode;
-        Peer* m_peer { nullptr };
-        Ref<SocketProvider> m_socketProvider;
+        ThreadSafeWeakPtr<Peer> m_peer;
+        const Ref<SocketProvider> m_socketProvider;
     };
 
     WEBCORE_EXPORT WorkerThreadableWebSocketChannel(WorkerGlobalScope&, WebSocketChannelClient&, const String& taskMode, SocketProvider&);
@@ -168,10 +166,10 @@ private:
     ResourceRequest clientHandshakeRequest(const CookieGetter&) const final { return m_handshakeRequest; }
     const ResourceResponse& serverHandshakeResponse() const final { return m_handshakeResponse; }
 
-    Ref<WorkerGlobalScope> m_workerGlobalScope;
-    Ref<ThreadableWebSocketChannelClientWrapper> m_workerClientWrapper;
+    const Ref<WorkerGlobalScope> m_workerGlobalScope;
+    const Ref<ThreadableWebSocketChannelClientWrapper> m_workerClientWrapper;
     RefPtr<Bridge> m_bridge;
-    Ref<SocketProvider> m_socketProvider;
+    const Ref<SocketProvider> m_socketProvider;
     ResourceRequest m_handshakeRequest;
     ResourceResponse m_handshakeResponse;
     WebSocketChannelIdentifier m_progressIdentifier;
