@@ -26,8 +26,6 @@
 #include "config.h"
 #include "OpenSSLUtilities.h"
 
-#if ENABLE(WEB_CRYPTO)
-
 #include "OpenSSLCryptoUniquePtr.h"
 
 namespace WebCore {
@@ -37,8 +35,9 @@ const EVP_MD* digestAlgorithm(CryptoAlgorithmIdentifier hashFunction)
     switch (hashFunction) {
     case CryptoAlgorithmIdentifier::SHA_1:
         return EVP_sha1();
-    case CryptoAlgorithmIdentifier::SHA_224:
-        return EVP_sha224();
+    case CryptoAlgorithmIdentifier::DEPRECATED_SHA_224:
+        RELEASE_ASSERT_NOT_REACHED_WITH_MESSAGE(sha224DeprecationMessage);
+        return EVP_sha256();
     case CryptoAlgorithmIdentifier::SHA_256:
         return EVP_sha256();
     case CryptoAlgorithmIdentifier::SHA_384:
@@ -64,10 +63,10 @@ std::optional<Vector<uint8_t>> calculateDigest(const EVP_MD* algorithm, const Ve
     if (EVP_DigestInit_ex(ctx.get(), algorithm, nullptr) != 1)
         return std::nullopt;
 
-    if (EVP_DigestUpdate(ctx.get(), message.data(), message.size()) != 1)
+    if (EVP_DigestUpdate(ctx.get(), message.span().data(), message.size()) != 1)
         return std::nullopt;
 
-    if (EVP_DigestFinal_ex(ctx.get(), digest.data(), nullptr) != 1)
+    if (EVP_DigestFinal_ex(ctx.get(), digest.mutableSpan().data(), nullptr) != 1)
         return std::nullopt;
 
     return digest;
@@ -76,7 +75,7 @@ std::optional<Vector<uint8_t>> calculateDigest(const EVP_MD* algorithm, const Ve
 Vector<uint8_t> convertToBytes(const BIGNUM* bignum)
 {
     Vector<uint8_t> bytes(BN_num_bytes(bignum));
-    BN_bn2bin(bignum, bytes.data());
+    BN_bn2bin(bignum, bytes.mutableSpan().data());
     return bytes;
 }
 
@@ -96,16 +95,40 @@ Vector<uint8_t> convertToBytesExpand(const BIGNUM* bignum, size_t minimumBufferS
         for (size_t i = 0; i < paddingLength; i++)
             bytes[i] = padding;
     }
-    BN_bn2bin(bignum, bytes.data() + paddingLength);
+    BN_bn2bin(bignum, bytes.mutableSpan().subspan(paddingLength).data());
     return bytes;
 }
 
 BIGNUMPtr convertToBigNumber(const Vector<uint8_t>& bytes)
 {
-    return BIGNUMPtr(BN_bin2bn(bytes.data(), bytes.size(), nullptr));
+    return BIGNUMPtr(BN_bin2bn(bytes.span().data(), bytes.size(), nullptr));
+}
+
+bool AESKey::setKey(const Vector<uint8_t>& key, int enc)
+{
+    size_t keySize = key.size() * 8;
+    if (keySize != 128 && keySize != 192 && keySize != 256)
+        return false;
+
+    if (enc == AES_ENCRYPT) {
+        if (AES_set_encrypt_key(key.span().data(), keySize, &m_key) < 0)
+            return false;
+        return true;
+    }
+
+    if (enc == AES_DECRYPT) {
+        if (AES_set_decrypt_key(key.span().data(), keySize, &m_key) < 0)
+            return false;
+        return true;
+    }
+
+    ASSERT_NOT_REACHED();
+    return false;
+}
+
+AESKey::~AESKey()
+{
+    memset(&m_key, 0, sizeof m_key);
 }
 
 } // namespace WebCore
-
-
-#endif // ENABLE(WEB_CRYPTO)
