@@ -33,8 +33,11 @@
 #include "DocumentInlines.h"
 #include "Element.h"
 #include "RenderStyleInlines.h"
+#include "StyleCustomProperty.h"
+#include "StyleExtractor.h"
 #include "StylePropertyShorthand.h"
 #include "StyleScope.h"
+#include <ranges>
 #include <wtf/KeyValuePair.h>
 
 namespace WebCore {
@@ -51,28 +54,27 @@ ComputedStylePropertyMapReadOnly::ComputedStylePropertyMapReadOnly(Element& elem
 
 RefPtr<CSSValue> ComputedStylePropertyMapReadOnly::propertyValue(CSSPropertyID propertyID) const
 {
-    return ComputedStyleExtractor(m_element.get()).propertyValue(propertyID, ComputedStyleExtractor::UpdateLayout::Yes, ComputedStyleExtractor::PropertyValueType::Computed);
+    return Style::Extractor { m_element.get() }.propertyValue(propertyID, Style::Extractor::UpdateLayout::Yes, Style::ExtractorState::PropertyValueType::Computed);
 }
 
 String ComputedStylePropertyMapReadOnly::shorthandPropertySerialization(CSSPropertyID propertyID) const
 {
-    auto value = propertyValue(propertyID);
-    return value ? value->cssText(CSS::defaultSerializationContext()) : String();
+    return Style::Extractor { m_element.get() }.propertyValueSerialization(propertyID, CSS::defaultSerializationContext(), Style::Extractor::UpdateLayout::Yes, Style::ExtractorState::PropertyValueType::Computed);
 }
 
 RefPtr<CSSValue> ComputedStylePropertyMapReadOnly::customPropertyValue(const AtomString& property) const
 {
-    return ComputedStyleExtractor(m_element.get()).customPropertyValue(property);
+    return Style::Extractor { m_element.get() }.customPropertyValue(property);
 }
 
 unsigned ComputedStylePropertyMapReadOnly::size() const
 {
     // https://drafts.css-houdini.org/css-typed-om-1/#dom-stylepropertymapreadonly-size
-    RefPtr element = protectedElement();
+    RefPtr element = m_element.get();
     if (!element)
         return 0;
 
-    ComputedStyleExtractor::updateStyleIfNeededForProperty(*element.get(), CSSPropertyCustom);
+    Style::Extractor::updateStyleIfNeededForProperty(*element.get(), CSSPropertyCustom);
 
     auto* style = element->computedStyle();
     if (!style)
@@ -83,7 +85,7 @@ unsigned ComputedStylePropertyMapReadOnly::size() const
 
 Vector<StylePropertyMapReadOnly::StylePropertyMapEntry> ComputedStylePropertyMapReadOnly::entries(ScriptExecutionContext*) const
 {
-    RefPtr element = protectedElement();
+    RefPtr element = m_element.get();
     if (!element)
         return { };
 
@@ -91,32 +93,41 @@ Vector<StylePropertyMapReadOnly::StylePropertyMapEntry> ComputedStylePropertyMap
     Vector<StylePropertyMapReadOnly::StylePropertyMapEntry> values;
 
     // Ensure custom property counts are correct.
-    ComputedStyleExtractor::updateStyleIfNeededForProperty(*element.get(), CSSPropertyCustom);
+    Style::Extractor::updateStyleIfNeededForProperty(*element.get(), CSSPropertyCustom);
 
     auto* style = element->computedStyle();
     if (!style)
         return values;
 
-    Ref document = element->protectedDocument();
+    Ref document = element->document();
     const auto& inheritedCustomProperties = style->inheritedCustomProperties();
     const auto& nonInheritedCustomProperties = style->nonInheritedCustomProperties();
     const auto& exposedComputedCSSPropertyIDs = document->exposedComputedCSSPropertyIDs();
     values.reserveInitialCapacity(exposedComputedCSSPropertyIDs.size() + inheritedCustomProperties.size() + nonInheritedCustomProperties.size());
 
-    ComputedStyleExtractor extractor { element.get() };
+    Style::Extractor computedStyleExtractor { element.get() };
     values.appendContainerWithMapping(exposedComputedCSSPropertyIDs, [&](auto propertyID) {
-        auto value = extractor.propertyValue(propertyID, ComputedStyleExtractor::UpdateLayout::No, ComputedStyleExtractor::PropertyValueType::Computed);
-        return makeKeyValuePair(nameString(propertyID), StylePropertyMapReadOnly::reifyValueToVector(WTFMove(value), propertyID, document));
+        auto value = computedStyleExtractor.propertyValue(propertyID, Style::Extractor::UpdateLayout::No, Style::ExtractorState::PropertyValueType::Computed);
+        return makeKeyValuePair(nameString(propertyID), StylePropertyMapReadOnly::reifyValueToVector(document, WTFMove(value), propertyID));
     });
 
     for (auto* map : { &nonInheritedCustomProperties, &inheritedCustomProperties }) {
         map->forEach([&](auto& it) {
-            values.append(makeKeyValuePair(it.key, StylePropertyMapReadOnly::reifyValueToVector(const_cast<CSSCustomPropertyValue*>(it.value.get()), std::nullopt, document)));
+            values.append(
+                makeKeyValuePair(
+                    it.key,
+                    StylePropertyMapReadOnly::reifyValueToVector(
+                        document,
+                        computedStyleExtractor.customPropertyValue(it.value->name()),
+                        std::nullopt
+                    )
+                )
+            );
             return IterationStatus::Continue;
         });
     }
 
-    std::sort(values.begin(), values.end(), [](const auto& a, const auto& b) {
+    std::ranges::sort(values, [](const auto& a, const auto& b) {
         const auto& nameA = a.key;
         const auto& nameB = b.key;
         if (nameA.startsWith("--"_s))
