@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2004-2020 Apple Inc. All rights reserved.
- * Copyright (C) 2010 Google Inc. All rights reserved.
+ * Copyright (C) 2004-2025 Apple Inc. All rights reserved.
+ * Copyright (C) 2010-2014 Google Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,33 +23,35 @@
 #include "FileInputType.h"
 
 #include "Chrome.h"
+#include "ContainerNodeInlines.h"
 #include "DOMFormData.h"
-#include "DeprecatedGlobalSettings.h"
 #include "DirectoryFileListCreator.h"
 #include "DragData.h"
-#include "ElementChildIterator.h"
+#include "ElementInlines.h"
 #include "ElementRareData.h"
 #include "Event.h"
 #include "File.h"
 #include "FileChooser.h"
 #include "FileList.h"
 #include "FormController.h"
-#include "Frame.h"
 #include "HTMLInputElement.h"
 #include "HTMLNames.h"
 #include "Icon.h"
 #include "InputTypeNames.h"
+#include "LocalFrame.h"
 #include "LocalizedStrings.h"
 #include "MIMETypeRegistry.h"
 #include "RenderFileUploadControl.h"
 #include "ScriptDisallowedScope.h"
 #include "Settings.h"
-#include "ShadowPseudoIds.h"
 #include "ShadowRoot.h"
+#include "UserAgentParts.h"
 #include "UserGestureIndicator.h"
+#include <wtf/FileHandle.h>
 #include <wtf/FileSystem.h>
-#include <wtf/IsoMallocInlines.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/TypeCasts.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/StringBuilder.h>
 
 #if PLATFORM(MAC)
@@ -58,57 +60,11 @@
 #endif
 
 namespace WebCore {
-class UploadButtonElement;
-}
 
-SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::UploadButtonElement)
-    static bool isType(const WebCore::Element& element) { return element.isUploadButton(); }
-    static bool isType(const WebCore::Node& node) { return is<WebCore::Element>(node) && isType(downcast<WebCore::Element>(node)); }
-SPECIALIZE_TYPE_TRAITS_END()
-
-namespace WebCore {
+WTF_MAKE_TZONE_ALLOCATED_IMPL(FileInputType);
 
 using namespace HTMLNames;
 
-// FIXME: This can likely be an HTMLDivElement.
-class UploadButtonElement final : public HTMLInputElement {
-    WTF_MAKE_ISO_ALLOCATED_INLINE(UploadButtonElement);
-public:
-    static Ref<UploadButtonElement> create(Document&);
-    static Ref<UploadButtonElement> createForMultiple(Document&);
-
-private:
-    static Ref<UploadButtonElement> createInternal(Document&, const String& value);
-    bool isUploadButton() const override { return true; }
-    
-    UploadButtonElement(Document&);
-};
-
-Ref<UploadButtonElement> UploadButtonElement::create(Document& document)
-{
-    return createInternal(document, fileButtonChooseFileLabel());
-}
-
-Ref<UploadButtonElement> UploadButtonElement::createForMultiple(Document& document)
-{
-    return createInternal(document, fileButtonChooseMultipleFilesLabel());
-}
-
-Ref<UploadButtonElement> UploadButtonElement::createInternal(Document& document, const String& value)
-{
-    auto button = adoptRef(*new UploadButtonElement(document));
-    ScriptDisallowedScope::EventAllowedScope eventAllowedScope { button };
-    button->setType(HTMLNames::buttonTag->localName());
-    button->setPseudo(ShadowPseudoIds::fileSelectorButton());
-    button->setValue(value);
-    return button;
-}
-
-UploadButtonElement::UploadButtonElement(Document& document)
-    : HTMLInputElement(inputTag, document, 0, false)
-{
-}
-
 FileInputType::FileInputType(HTMLInputElement& element)
     : BaseClickableWithKeyInputType(Type::File, element)
     , m_fileList(FileList::create())
@@ -118,21 +74,22 @@ FileInputType::FileInputType(HTMLInputElement& element)
 
 FileInputType::~FileInputType()
 {
-    if (m_fileChooser)
-        m_fileChooser->invalidate();
+    if (RefPtr fileChooser = m_fileChooser)
+        fileChooser->invalidate();
 
     if (m_fileIconLoader)
         m_fileIconLoader->invalidate();
 }
 
-Vector<FileChooserFileInfo> FileInputType::filesFromFormControlState(const FormControlState& state)
+std::pair<Vector<FileChooserFileInfo>, String> FileInputType::filesFromFormControlState(const FormControlState& state)
 {
     Vector<FileChooserFileInfo> files;
-    size_t size = state.size();
+    size_t size = state.size() - 1;
     files.reserveInitialCapacity(size / 2);
     for (size_t i = 0; i < size; i += 2)
-        files.uncheckedAppend({ state[i], { }, state[i + 1] });
-    return files;
+        files.append({ state[i], { }, state[i + 1] });
+
+    return { files, state[size] };
 }
 
 const AtomString& FileInputType::formControlType() const
@@ -145,35 +102,37 @@ FormControlState FileInputType::saveFormControlState() const
     if (m_fileList->isEmpty())
         return { };
 
-    auto length = Checked<size_t>(m_fileList->files().size()) * Checked<size_t>(2);
+    auto length = Checked<size_t>(m_fileList->files().size()) * Checked<size_t>(2) + Checked<size_t>(1);
 
     Vector<AtomString> stateVector;
     stateVector.reserveInitialCapacity(length);
     for (auto& file : m_fileList->files()) {
-        stateVector.uncheckedAppend(AtomString { file->path() });
-        stateVector.uncheckedAppend(AtomString { file->name() });
+        stateVector.append(AtomString { file->path() });
+        stateVector.append(AtomString { file->name() });
     }
+    stateVector.append(AtomString { m_displayString });
     return FormControlState { WTFMove(stateVector) };
 }
 
 void FileInputType::restoreFormControlState(const FormControlState& state)
 {
-    filesChosen(filesFromFormControlState(state));
+    auto [files, displayString] = filesFromFormControlState(state);
+    filesChosen(files, displayString);
 }
 
 bool FileInputType::appendFormData(DOMFormData& formData) const
 {
     ASSERT(element());
-    RefPtr fileList = element()->files();
-    ASSERT(fileList);
+    Ref element = *this->element();
+    Ref fileList = m_fileList;
 
-    auto name = element()->name();
+    auto name = element->name();
 
     // If no filename at all is entered, return successful but empty.
     // Null would be more logical, but Netscape posts an empty file. Argh.
     if (fileList->isEmpty()) {
-        auto* document = element() ? &element()->document() : nullptr;
-        auto file = File::create(document, Blob::create(document, { }, defaultMIMEType()), emptyString());
+        Ref document = element->document();
+        auto file = File::create(document.ptr(), Blob::create(document.ptr(), { }, defaultMIMEType()), emptyString());
         formData.append(name, file);
         return true;
     }
@@ -192,15 +151,14 @@ bool FileInputType::valueMissing(const String& value) const
 String FileInputType::valueMissingText() const
 {
     ASSERT(element());
-    return element()->multiple() ? validationMessageValueMissingForMultipleFileText() : validationMessageValueMissingForFileText();
+    return protectedElement()->multiple() ? validationMessageValueMissingForMultipleFileText() : validationMessageValueMissingForFileText();
 }
 
 void FileInputType::handleDOMActivateEvent(Event& event)
 {
     ASSERT(element());
-    auto& input = *element();
 
-    if (input.isDisabledFormControl())
+    if (protectedElement()->isDisabledFormControl())
         return;
 
     if (!UserGestureIndicator::processingUserGesture())
@@ -213,11 +171,9 @@ void FileInputType::handleDOMActivateEvent(Event& event)
 void FileInputType::showPicker()
 {
     ASSERT(element());
-    auto& input = *element();
-
     if (auto* chrome = this->chrome()) {
         applyFileChooserSettings();
-        chrome->runOpenPanel(*input.document().frame(), *m_fileChooser);
+        chrome->runOpenPanel(*element()->document().protectedFrame(), *protectedFileChooser());
     }
 }
 
@@ -229,7 +185,8 @@ bool FileInputType::allowsShowPickerAcrossFrames()
 RenderPtr<RenderElement> FileInputType::createInputRenderer(RenderStyle&& style)
 {
     ASSERT(element());
-    return createRenderer<RenderFileUploadControl>(*element(), WTFMove(style));
+    // FIXME: https://github.com/llvm/llvm-project/pull/142471 Moving style is not unsafe.
+    SUPPRESS_UNCOUNTED_ARG return createRenderer<RenderFileUploadControl>(*protectedElement(), WTFMove(style));
 }
 
 bool FileInputType::canSetStringValue() const
@@ -248,16 +205,21 @@ String FileInputType::firstElementPathForInputValue() const
     // decided to try to parse the value by looking for backslashes
     // (because that's what Windows file paths use). To be compatible
     // with that code, we make up a fake path for the file.
-    return makeString("C:\\fakepath\\", m_fileList->file(0).name());
+    return makeString("C:\\fakepath\\"_s, protectedFiles()->file(0).name());
 }
 
-void FileInputType::setValue(const String&, bool, TextFieldEventBehavior, TextControlSetValueSelection)
+void FileInputType::setValue(const String&, bool valueChanged, TextFieldEventBehavior, TextControlSetValueSelection)
 {
     // FIXME: Should we clear the file list, or replace it with a new empty one here? This is observable from JavaScript through custom properties.
-    m_fileList->clear();
+    if (!valueChanged)
+        return;
+    
+    protectedFiles()->clear();
     m_icon = nullptr;
     ASSERT(element());
-    element()->invalidateStyleForSubtree();
+    Ref element = *this->element();
+    element->invalidateStyleForSubtree();
+    element->updateValidity();
 }
 
 void FileInputType::createShadowSubtree()
@@ -266,32 +228,41 @@ void FileInputType::createShadowSubtree()
     ASSERT(element());
     ASSERT(element()->shadowRoot());
 
-    auto button = element()->multiple() ? UploadButtonElement::createForMultiple(element()->document()) : UploadButtonElement::create(element()->document());
-    ScriptDisallowedScope::EventAllowedScope eventAllowedScope { *element()->userAgentShadowRoot() };
-    element()->userAgentShadowRoot()->appendChild(ContainerNode::ChildChange::Source::Parser, button);
+    Ref element = *this->element();
+    Ref button = HTMLInputElement::create(inputTag, element->protectedDocument(), nullptr, false);
+    {
+        ScriptDisallowedScope::EventAllowedScope eventAllowedScopeBeforeAppend { button };
+        button->setAttributeWithoutSynchronization(typeAttr, InputTypeNames::button());
+        button->setUserAgentPart(UserAgentParts::fileSelectorButton());
+        button->setValue(element->multiple() ? fileButtonChooseMultipleFilesLabel() : fileButtonChooseFileLabel());
+    }
+    Ref shadowRoot = *element->userAgentShadowRoot();
+    ScriptDisallowedScope::EventAllowedScope eventAllowedScope { shadowRoot };
+    shadowRoot->appendChild(ContainerNode::ChildChange::Source::Parser, button);
     disabledStateChanged();
 }
 
+static RefPtr<HTMLInputElement> fileSelectorButton(const Element& element)
+{
+    RefPtr root = element.userAgentShadowRoot();
+    return root ? downcast<HTMLInputElement>(root->firstChild()) : nullptr;
+}
+
 void FileInputType::disabledStateChanged()
 {
     ASSERT(element());
 
-    auto root = element()->userAgentShadowRoot();
-    if (!root)
-        return;
-    
-    if (RefPtr button = childrenOfType<UploadButtonElement>(*root).first())
-        button->setBooleanAttribute(disabledAttr, element()->isDisabledFormControl());
+    Ref element = *this->element();
+    if (RefPtr button = fileSelectorButton(element))
+        button->setBooleanAttribute(disabledAttr, element->isDisabledFormControl());
 }
 
 void FileInputType::attributeChanged(const QualifiedName& name)
 {
     if (name == multipleAttr) {
-        if (auto* element = this->element()) {
-            if (auto root = element->userAgentShadowRoot()) {
-                if (RefPtr button = childrenOfType<UploadButtonElement>(*root).first())
-                    button->setValue(element->multiple() ? fileButtonChooseMultipleFilesLabel() : fileButtonChooseFileLabel());
-            }
+        if (RefPtr element = this->element()) {
+            if (RefPtr button = fileSelectorButton(*element))
+                button->setValue(element->multiple() ? fileButtonChooseMultipleFilesLabel() : fileButtonChooseFileLabel());
         }
     }
     BaseClickableWithKeyInputType::attributeChanged(name);
@@ -322,34 +293,40 @@ void FileInputType::requestIcon(const Vector<String>& paths)
 FileChooserSettings FileInputType::fileChooserSettings() const
 {
     ASSERT(element());
-    auto& input = *element();
+    Ref element = *this->element();
 
     FileChooserSettings settings;
     settings.allowsDirectories = allowsDirectories();
-    settings.allowsMultipleFiles = input.hasAttributeWithoutSynchronization(multipleAttr);
-    settings.acceptMIMETypes = input.acceptMIMETypes();
-    settings.acceptFileExtensions = input.acceptFileExtensions();
-    settings.selectedFiles = m_fileList->paths();
+    settings.allowsMultipleFiles = element->hasAttributeWithoutSynchronization(multipleAttr);
+    settings.acceptMIMETypes = element->acceptMIMETypes();
+    settings.acceptFileExtensions = element->acceptFileExtensions();
+    settings.selectedFiles = protectedFiles()->paths();
 #if ENABLE(MEDIA_CAPTURE)
-    settings.mediaCaptureType = input.mediaCaptureType();
+    settings.mediaCaptureType = element->mediaCaptureType();
 #endif
     return settings;
 }
 
 void FileInputType::applyFileChooserSettings()
 {
-    if (m_fileChooser)
-        m_fileChooser->invalidate();
+    if (RefPtr fileChooser = m_fileChooser)
+        fileChooser->invalidate();
 
     m_fileChooser = FileChooser::create(*this, fileChooserSettings());
 }
 
 bool FileInputType::allowsDirectories() const
 {
-    if (!DeprecatedGlobalSettings::directoryUploadEnabled())
-        return false;
     ASSERT(element());
-    return element()->hasAttributeWithoutSynchronization(webkitdirectoryAttr);
+    Ref element = *this->element();
+    if (!element->protectedDocument()->settings().directoryUploadEnabled())
+        return false;
+    return element->hasAttributeWithoutSynchronization(webkitdirectoryAttr);
+}
+
+bool FileInputType::dirAutoUsesValue() const
+{
+    return false;
 }
 
 void FileInputType::setFiles(RefPtr<FileList>&& files, WasSetByJavaScript wasSetByJavaScript)
@@ -363,7 +340,7 @@ void FileInputType::setFiles(RefPtr<FileList>&& files, RequestIcon shouldRequest
         return;
 
     ASSERT(element());
-    Ref<HTMLInputElement> protectedInputElement(*element());
+    Ref element = *this->element();
 
     unsigned length = files->length();
 
@@ -371,8 +348,9 @@ void FileInputType::setFiles(RefPtr<FileList>&& files, RequestIcon shouldRequest
     if (length != m_fileList->length())
         pathsChanged = true;
     else {
+        Ref currentFiles = m_fileList;
         for (unsigned i = 0; i < length; ++i) {
-            if (files->file(i).path() != m_fileList->file(i).path()) {
+            if (files->file(i).path() != currentFiles->file(i).path() || !FileSystem::fileIDsAreEqual(files->file(i).fileID(), currentFiles->file(i).fileID())) {
                 pathsChanged = true;
                 break;
             }
@@ -381,14 +359,14 @@ void FileInputType::setFiles(RefPtr<FileList>&& files, RequestIcon shouldRequest
 
     m_fileList = files.releaseNonNull();
 
-    protectedInputElement->setFormControlValueMatchesRenderer(true);
-    protectedInputElement->updateValidity();
+    element->setFormControlValueMatchesRenderer(true);
+    element->updateValidity();
 
     if (shouldRequestIcon == RequestIcon::Yes)
-        requestIcon(m_fileList->paths());
+        requestIcon(protectedFiles()->paths());
 
-    if (protectedInputElement->renderer())
-        protectedInputElement->renderer()->repaint();
+    if (CheckedPtr renderer = element->renderer())
+        renderer->repaint();
 
     if (wasSetByJavaScript == WasSetByJavaScript::Yes)
         return;
@@ -396,10 +374,13 @@ void FileInputType::setFiles(RefPtr<FileList>&& files, RequestIcon shouldRequest
     if (pathsChanged) {
         // This call may cause destruction of this instance.
         // input instance is safe since it is ref-counted.
-        protectedInputElement->dispatchInputEvent();
-        protectedInputElement->dispatchChangeEvent();
-    }
-    protectedInputElement->setChangedSinceLastFormControlChangeEvent(false);
+        element->dispatchInputEvent();
+        element->dispatchChangeEvent();
+    } else
+        element->dispatchCancelEvent();
+
+    element->setChangedSinceLastFormControlChangeEvent(false);
+    element->setInteractedWithSinceLastFormSubmitEvent(true);
 }
 
 void FileInputType::filesChosen(const Vector<FileChooserFileInfo>& paths, const String& displayString, Icon* icon)
@@ -407,25 +388,30 @@ void FileInputType::filesChosen(const Vector<FileChooserFileInfo>& paths, const
     if (!displayString.isEmpty())
         m_displayString = displayString;
 
-    if (m_directoryFileListCreator)
-        m_directoryFileListCreator->cancel();
+    if (RefPtr creator = m_directoryFileListCreator)
+        creator->cancel();
 
-    auto* document = element() ? &element()->document() : nullptr;
+    RefPtr document = element() ? &element()->document() : nullptr;
     if (!allowsDirectories()) {
         auto files = paths.map([document](auto& fileInfo) {
-            return File::create(document, fileInfo.path, fileInfo.replacementPath, fileInfo.displayName);
+            std::optional<FileSystem::PlatformFileID> fileID;
+            if (auto handle = FileSystem::openFile(fileInfo.path, FileSystem::FileOpenMode::Read); handle)
+                fileID = handle.id();
+            return File::create(document.get(), fileInfo.path, fileInfo.replacementPath, fileInfo.displayName, fileID);
         });
         didCreateFileList(FileList::create(WTFMove(files)), icon);
         return;
     }
 
-    m_directoryFileListCreator = DirectoryFileListCreator::create([this, weakThis = WeakPtr { *this }, icon = RefPtr { icon }](Ref<FileList>&& fileList) mutable {
+    Ref creator = DirectoryFileListCreator::create([weakThis = WeakPtr { *this }, icon = RefPtr { icon }](Ref<FileList>&& fileList) mutable {
         ASSERT(isMainThread());
-        if (!weakThis)
+        RefPtr protectedThis = weakThis.get();
+        if (!protectedThis)
             return;
-        didCreateFileList(WTFMove(fileList), WTFMove(icon));
+        protectedThis->didCreateFileList(WTFMove(fileList), WTFMove(icon));
     });
-    m_directoryFileListCreator->start(document, paths);
+    creator->start(document.get(), paths);
+    m_directoryFileListCreator = WTFMove(creator);
 }
 
 void FileInputType::filesChosen(const Vector<String>& paths, const Vector<String>& replacementPaths)
@@ -433,17 +419,21 @@ void FileInputType::filesChosen(const Vector<String>& paths, const Vector<String
     ASSERT(element());
     ASSERT(!paths.isEmpty());
 
-    size_t size = element()->hasAttributeWithoutSynchronization(multipleAttr) ? paths.size() : 1;
+    size_t size = protectedElement()->hasAttributeWithoutSynchronization(multipleAttr) ? paths.size() : 1;
 
-    Vector<FileChooserFileInfo> files;
-    files.reserveInitialCapacity(size);
-
-    for (size_t i = 0; i < size; ++i)
-        files.uncheckedAppend({ paths[i], i < replacementPaths.size() ? replacementPaths[i] : nullString(), { } });
+    Vector<FileChooserFileInfo> files(size, [&](size_t i) {
+        return FileChooserFileInfo { paths[i], i < replacementPaths.size() ? replacementPaths[i] : nullString(), { } };
+    });
 
     filesChosen(WTFMove(files));
 }
 
+void FileInputType::fileChoosingCancelled()
+{
+    ASSERT(element());
+    protectedElement()->dispatchCancelEvent();
+}
+
 void FileInputType::didCreateFileList(Ref<FileList>&& fileList, RefPtr<Icon>&& icon)
 {
     Ref protectedThis { *this };
@@ -468,7 +458,7 @@ void FileInputType::iconLoaded(RefPtr<Icon>&& icon)
 
     m_icon = WTFMove(icon);
     ASSERT(element());
-    if (auto* renderer = element()->renderer())
+    if (CheckedPtr renderer = element()->renderer())
         renderer->repaint();
 }
 
@@ -494,13 +484,13 @@ bool FileInputType::receiveDroppedFilesWithImageTranscoding(const Vector<String>
         protectedThis->filesChosen(paths, replacementPaths);
     };
 
-    sharedImageTranscodingQueue().dispatch([callFilesChosen = WTFMove(callFilesChosen), transcodingPaths = crossThreadCopy(WTFMove(transcodingPaths)), transcodingUTI = WTFMove(transcodingUTI).isolatedCopy(), transcodingExtension = WTFMove(transcodingExtension).isolatedCopy()]() mutable {
+    sharedImageTranscodingQueueSingleton().dispatch([callFilesChosen = WTFMove(callFilesChosen), transcodingPaths = crossThreadCopy(WTFMove(transcodingPaths)), transcodingUTI = WTFMove(transcodingUTI).isolatedCopy(), transcodingExtension = WTFMove(transcodingExtension).isolatedCopy()]() mutable {
         ASSERT(!RunLoop::isMain());
 
         auto replacementPaths = transcodeImages(transcodingPaths, transcodingUTI, transcodingExtension);
         ASSERT(transcodingPaths.size() == replacementPaths.size());
 
-        RunLoop::main().dispatch([callFilesChosen = WTFMove(callFilesChosen), replacementPaths = crossThreadCopy(WTFMove(replacementPaths))] {
+        RunLoop::mainSingleton().dispatch([callFilesChosen = WTFMove(callFilesChosen), replacementPaths = crossThreadCopy(WTFMove(replacementPaths))] {
             callFilesChosen(replacementPaths);
         });
     });
@@ -536,14 +526,15 @@ String FileInputType::defaultToolTip() const
     unsigned listSize = m_fileList->length();
     if (!listSize) {
         ASSERT(element());
-        if (element()->multiple())
+        if (protectedElement()->multiple())
             return fileButtonNoFilesSelectedLabel();
         return fileButtonNoFileSelectedLabel();
     }
 
     StringBuilder names;
+    Ref fileList = m_fileList;
     for (unsigned i = 0; i < listSize; ++i) {
-        names.append(m_fileList->file(i).name());
+        names.append(fileList->file(i).name());
         if (i != listSize - 1)
             names.append('\n');
     }
