@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2018 Apple Inc.  All rights reserved.
+ * Copyright (C) 2016-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,7 +27,7 @@
 #include "ResourceLoadStatistics.h"
 
 #include "KeyedCoding.h"
-#include "PublicSuffix.h"
+#include "PublicSuffixStore.h"
 #include <wtf/MainThread.h>
 #include <wtf/text/ASCIILiteral.h>
 #include <wtf/text/StringBuilder.h>
@@ -43,8 +43,8 @@ static void encodeHashSet(KeyedEncoder& encoder, const String& label,  const Str
 {
     if (hashSet.isEmpty())
         return;
-    
-    encoder.encodeObjects(label, hashSet.begin(), hashSet.end(), [&key](KeyedEncoder& encoderInner, const RegistrableDomain& domain) {
+
+    encoder.encodeObjects(label, hashSet, [&key](KeyedEncoder& encoderInner, const RegistrableDomain& domain) {
         encoderInner.encodeString(key, domain.string());
     });
 }
@@ -54,7 +54,7 @@ static void encodeOptionSet(KeyedEncoder& encoder, const String& label, const Op
 {
     if (optionSet.isEmpty())
         return;
-    
+
     uint64_t optionSetBitMask = optionSet.toRaw();
     encoder.encodeUInt64(label, optionSetBitMask);
 }
@@ -64,23 +64,23 @@ static void encodeHashSet(KeyedEncoder& encoder, const String& label,  const Str
 {
     if (hashSet.isEmpty())
         return;
-    
-    encoder.encodeObjects(label, hashSet.begin(), hashSet.end(), [&key](KeyedEncoder& encoderInner, const String& origin) {
+
+    encoder.encodeObjects(label, hashSet, [&key](KeyedEncoder& encoderInner, const String& origin) {
         encoderInner.encodeString(key, origin);
     });
 }
 
 static void encodeFontHashSet(KeyedEncoder& encoder, const String& label, const HashSet<String>& hashSet)
 {
-    encodeHashSet(encoder, label, "font", hashSet);
+    encodeHashSet(encoder, label, "font"_s, hashSet);
 }
-    
+
 static void encodeCanvasActivityRecord(KeyedEncoder& encoder, const String& label, const CanvasActivityRecord& canvasActivityRecord)
 {
     encoder.encodeObject(label, canvasActivityRecord, [] (KeyedEncoder& encoderInner, const CanvasActivityRecord& canvasActivityRecord) {
-        encoderInner.encodeBool("wasDataRead", canvasActivityRecord.wasDataRead);
-        encoderInner.encodeObjects("textWritten", canvasActivityRecord.textWritten.begin(), canvasActivityRecord.textWritten.end(), [] (KeyedEncoder& encoderInner2, const String& text) {
-            encoderInner2.encodeString("text", text);
+        encoderInner.encodeBool("wasDataRead"_s, canvasActivityRecord.wasDataRead);
+        encoderInner.encodeObjects("textWritten"_s, canvasActivityRecord.textWritten, [] (KeyedEncoder& encoderInner2, const String& text) {
+            encoderInner2.encodeString("text"_s, text);
         });
     });
 }
@@ -103,13 +103,13 @@ void ResourceLoadStatistics::encode(KeyedEncoder& encoder) const
     // Top frame stats
     encodeHashSet(encoder, "topFrameUniqueRedirectsTo"_s, "domain"_s, topFrameUniqueRedirectsTo);
     encodeHashSet(encoder, "topFrameUniqueRedirectsFrom"_s, "domain"_s, topFrameUniqueRedirectsFrom);
-    encodeHashSet(encoder, "topFrameLinkDecorationsFrom"_s, "domain", topFrameLinkDecorationsFrom);
+    encodeHashSet(encoder, "topFrameLinkDecorationsFrom"_s, "domain"_s, topFrameLinkDecorationsFrom);
     encoder.encodeBool("gotLinkDecorationFromPrevalentResource"_s, gotLinkDecorationFromPrevalentResource);
-    encodeHashSet(encoder, "topFrameLoadedThirdPartyScripts"_s, "domain", topFrameLoadedThirdPartyScripts);
+    encodeHashSet(encoder, "topFrameLoadedThirdPartyScripts"_s, "domain"_s, topFrameLoadedThirdPartyScripts);
 
     // Subframe stats
     encodeHashSet(encoder, "subframeUnderTopFrameDomains"_s, "domain"_s, subframeUnderTopFrameDomains);
-    
+
     // Subresource stats
     encodeHashSet(encoder, "subresourceUnderTopFrameDomains"_s, "domain"_s, subresourceUnderTopFrameDomains);
     encodeHashSet(encoder, "subresourceUniqueRedirectsTo"_s, "domain"_s, subresourceUniqueRedirectsTo);
@@ -124,12 +124,12 @@ void ResourceLoadStatistics::encode(KeyedEncoder& encoder) const
     encoder.encodeUInt32("timesAccessedAsFirstPartyDueToStorageAccessAPI"_s, timesAccessedAsFirstPartyDueToStorageAccessAPI);
 
 #if ENABLE(WEB_API_STATISTICS)
-    encodeFontHashSet(encoder, "fontsFailedToLoad", fontsFailedToLoad);
-    encodeFontHashSet(encoder, "fontsSuccessfullyLoaded", fontsSuccessfullyLoaded);
-    encodeHashSet(encoder, "topFrameRegistrableDomainsWhichAccessedWebAPIs", "domain", topFrameRegistrableDomainsWhichAccessedWebAPIs);
-    encodeCanvasActivityRecord(encoder, "canvasActivityRecord", canvasActivityRecord);
-    encodeOptionSet(encoder, "navigatorFunctionsAccessedBitMask", navigatorFunctionsAccessed);
-    encodeOptionSet(encoder, "screenFunctionsAccessedBitMask", screenFunctionsAccessed);
+    encodeFontHashSet(encoder, "fontsFailedToLoad"_s, fontsFailedToLoad);
+    encodeFontHashSet(encoder, "fontsSuccessfullyLoaded"_s, fontsSuccessfullyLoaded);
+    encodeHashSet(encoder, "topFrameRegistrableDomainsWhichAccessedWebAPIs"_s, "domain"_s, topFrameRegistrableDomainsWhichAccessedWebAPIs);
+    encodeCanvasActivityRecord(encoder, "canvasActivityRecord"_s, canvasActivityRecord);
+    encodeOptionSet(encoder, "navigatorFunctionsAccessedBitMask"_s, navigatorFunctionsAccessed);
+    encodeOptionSet(encoder, "screenFunctionsAccessedBitMask"_s, screenFunctionsAccessed);
 #endif
 }
 
@@ -138,11 +138,11 @@ static void decodeHashCountedSet(KeyedDecoder& decoder, const String& label, Has
     Vector<String> ignored;
 IGNORE_WARNINGS_BEGIN("unused-result")
     decoder.decodeObjects(label, ignored, [&hashCountedSet](KeyedDecoder& decoderInner, String& domain) {
-        if (!decoderInner.decodeString("origin", domain))
+        if (!decoderInner.decodeString("origin"_s, domain))
             return false;
-        
+
         unsigned count;
-        if (!decoderInner.decodeUInt32("count", count))
+        if (!decoderInner.decodeUInt32("count"_s, count))
             return false;
 
         hashCountedSet.add(RegistrableDomain::uncheckedCreateFromRegistrableDomainString(domain), count);
@@ -158,7 +158,7 @@ IGNORE_WARNINGS_BEGIN("unused-result")
     decoder.decodeObjects(label, ignored, [&hashSet, &key](KeyedDecoder& decoderInner, String& domain) {
         if (!decoderInner.decodeString(key, domain))
             return false;
-        
+
         hashSet.add(RegistrableDomain::uncheckedCreateFromRegistrableDomainString(domain));
         return true;
     });
@@ -181,7 +181,7 @@ IGNORE_WARNINGS_BEGIN("unused-result")
     decoder.decodeObjects(label, ignore, [&hashSet, &key](KeyedDecoder& decoderInner, String& origin) {
         if (!decoderInner.decodeString(key, origin))
             return false;
-        
+
         hashSet.add(origin);
         return true;
     });
@@ -190,18 +190,18 @@ IGNORE_WARNINGS_END
 
 static void decodeFontHashSet(KeyedDecoder& decoder, const String& label, HashSet<String>& hashSet)
 {
-    decodeHashSet(decoder, label, "font", hashSet);
+    decodeHashSet(decoder, label, "font"_s, hashSet);
 }
-    
+
 static void decodeCanvasActivityRecord(KeyedDecoder& decoder, const String& label, CanvasActivityRecord& canvasActivityRecord)
 {
 IGNORE_WARNINGS_BEGIN("unused-result")
     decoder.decodeObject(label, canvasActivityRecord, [] (KeyedDecoder& decoderInner, CanvasActivityRecord& canvasActivityRecord) {
-        if (!decoderInner.decodeBool("wasDataRead", canvasActivityRecord.wasDataRead))
+        if (!decoderInner.decodeBool("wasDataRead"_s, canvasActivityRecord.wasDataRead))
             return false;
         Vector<String> ignore;
-        decoderInner.decodeObjects("textWritten", ignore, [&canvasActivityRecord] (KeyedDecoder& decoderInner2, String& text) {
-            if (!decoderInner2.decodeString("text", text))
+        decoderInner.decodeObjects("textWritten"_s, ignore, [&canvasActivityRecord] (KeyedDecoder& decoderInner2, String& text) {
+            if (!decoderInner2.decodeString("text"_s, text))
                 return false;
             canvasActivityRecord.textWritten.add(text);
             return true;
@@ -216,81 +216,81 @@ bool ResourceLoadStatistics::decode(KeyedDecoder& decoder, unsigned modelVersion
 {
     String registrableDomainAsString;
     if (modelVersion >= 15) {
-        if (!decoder.decodeString("PrevalentResourceDomain", registrableDomainAsString))
+        if (!decoder.decodeString("PrevalentResourceDomain"_s, registrableDomainAsString))
             return false;
     } else {
-        if (!decoder.decodeString("PrevalentResourceOrigin", registrableDomainAsString))
+        if (!decoder.decodeString("PrevalentResourceOrigin"_s, registrableDomainAsString))
             return false;
     }
     registrableDomain = RegistrableDomain::uncheckedCreateFromRegistrableDomainString(registrableDomainAsString);
 
     // User interaction
-    if (!decoder.decodeBool("hadUserInteraction", hadUserInteraction))
+    if (!decoder.decodeBool("hadUserInteraction"_s, hadUserInteraction))
         return false;
 
     // Storage access
     if (modelVersion >= 15)
-        decodeHashSet(decoder, "storageAccessUnderTopFrameDomains", "domain", storageAccessUnderTopFrameDomains);
+        decodeHashSet(decoder, "storageAccessUnderTopFrameDomains"_s, "domain"_s, storageAccessUnderTopFrameDomains);
     else
-        decodeHashSet(decoder, "storageAccessUnderTopFrameOrigins", "origin", storageAccessUnderTopFrameDomains);
+        decodeHashSet(decoder, "storageAccessUnderTopFrameOrigins"_s, "origin"_s, storageAccessUnderTopFrameDomains);
 
     // Top frame stats
     if (modelVersion >= 15) {
-        decodeHashSet(decoder, "topFrameUniqueRedirectsTo", "domain", topFrameUniqueRedirectsTo);
-        decodeHashSet(decoder, "topFrameUniqueRedirectsFrom", "domain", topFrameUniqueRedirectsFrom);
+        decodeHashSet(decoder, "topFrameUniqueRedirectsTo"_s, "domain"_s, topFrameUniqueRedirectsTo);
+        decodeHashSet(decoder, "topFrameUniqueRedirectsFrom"_s, "domain"_s, topFrameUniqueRedirectsFrom);
     } else if (modelVersion >= 11) {
         HashCountedSet<RegistrableDomain> topFrameUniqueRedirectsToCounted;
-        decodeHashCountedSet(decoder, "topFrameUniqueRedirectsTo", topFrameUniqueRedirectsToCounted);
+        decodeHashCountedSet(decoder, "topFrameUniqueRedirectsTo"_s, topFrameUniqueRedirectsToCounted);
         for (auto& domain : topFrameUniqueRedirectsToCounted.values())
             topFrameUniqueRedirectsTo.add(domain);
-        
+
         HashCountedSet<RegistrableDomain> topFrameUniqueRedirectsFromCounted;
-        decodeHashCountedSet(decoder, "topFrameUniqueRedirectsFrom", topFrameUniqueRedirectsFromCounted);
+        decodeHashCountedSet(decoder, "topFrameUniqueRedirectsFrom"_s, topFrameUniqueRedirectsFromCounted);
         for (auto& domain : topFrameUniqueRedirectsFromCounted.values())
             topFrameUniqueRedirectsFrom.add(domain);
     }
 
     if (modelVersion >= 16) {
-        decodeHashSet(decoder, "topFrameLinkDecorationsFrom", "domain", topFrameLinkDecorationsFrom);
-        if (!decoder.decodeBool("gotLinkDecorationFromPrevalentResource", gotLinkDecorationFromPrevalentResource))
+        decodeHashSet(decoder, "topFrameLinkDecorationsFrom"_s, "domain"_s, topFrameLinkDecorationsFrom);
+        if (!decoder.decodeBool("gotLinkDecorationFromPrevalentResource"_s, gotLinkDecorationFromPrevalentResource))
             return false;
     }
 
     if (modelVersion >= 17) {
         HashCountedSet<RegistrableDomain> topFrameLoadedThirdPartyScriptsCounted;
-        decodeHashCountedSet(decoder, "topFrameLoadedThirdPartyScripts", topFrameLoadedThirdPartyScriptsCounted);
+        decodeHashCountedSet(decoder, "topFrameLoadedThirdPartyScripts"_s, topFrameLoadedThirdPartyScriptsCounted);
         for (auto& domain : topFrameLoadedThirdPartyScriptsCounted.values())
             topFrameLoadedThirdPartyScripts.add(domain);
     }
 
     // Subframe stats
     if (modelVersion >= 15)
-        decodeHashSet(decoder, "subframeUnderTopFrameDomains", "domain", subframeUnderTopFrameDomains);
+        decodeHashSet(decoder, "subframeUnderTopFrameDomains"_s, "domain"_s, subframeUnderTopFrameDomains);
     else if (modelVersion >= 14) {
         HashCountedSet<RegistrableDomain> subframeUnderTopFrameDomainsCounted;
-        decodeHashCountedSet(decoder, "subframeUnderTopFrameOrigins", subframeUnderTopFrameDomainsCounted);
+        decodeHashCountedSet(decoder, "subframeUnderTopFrameOrigins"_s, subframeUnderTopFrameDomainsCounted);
         for (auto& domain : subframeUnderTopFrameDomainsCounted.values())
             subframeUnderTopFrameDomains.add(domain);
     }
 
     // Subresource stats
     if (modelVersion >= 15) {
-        decodeHashSet(decoder, "subresourceUnderTopFrameDomains", "domain", subresourceUnderTopFrameDomains);
-        decodeHashSet(decoder, "subresourceUniqueRedirectsTo", "domain", subresourceUniqueRedirectsTo);
-        decodeHashSet(decoder, "subresourceUniqueRedirectsFrom", "domain", subresourceUniqueRedirectsFrom);
+        decodeHashSet(decoder, "subresourceUnderTopFrameDomains"_s, "domain"_s, subresourceUnderTopFrameDomains);
+        decodeHashSet(decoder, "subresourceUniqueRedirectsTo"_s, "domain"_s, subresourceUniqueRedirectsTo);
+        decodeHashSet(decoder, "subresourceUniqueRedirectsFrom"_s, "domain"_s, subresourceUniqueRedirectsFrom);
     } else {
         HashCountedSet<RegistrableDomain> subresourceUnderTopFrameDomainsCounted;
-        decodeHashCountedSet(decoder, "subresourceUnderTopFrameOrigins", subresourceUnderTopFrameDomainsCounted);
+        decodeHashCountedSet(decoder, "subresourceUnderTopFrameOrigins"_s, subresourceUnderTopFrameDomainsCounted);
         for (auto& domain : subresourceUnderTopFrameDomainsCounted.values())
             subresourceUnderTopFrameDomains.add(domain);
 
         HashCountedSet<RegistrableDomain> subresourceUniqueRedirectsToCounted;
-        decodeHashCountedSet(decoder, "subresourceUniqueRedirectsTo", subresourceUniqueRedirectsToCounted);
+        decodeHashCountedSet(decoder, "subresourceUniqueRedirectsTo"_s, subresourceUniqueRedirectsToCounted);
         for (auto& domain : subresourceUniqueRedirectsToCounted.values())
             subresourceUniqueRedirectsTo.add(domain);
         if (modelVersion >= 11) {
             HashCountedSet<RegistrableDomain> subresourceUniqueRedirectsFromCounted;
-            decodeHashCountedSet(decoder, "subresourceUniqueRedirectsFrom", subresourceUniqueRedirectsFromCounted);
+            decodeHashCountedSet(decoder, "subresourceUniqueRedirectsFrom"_s, subresourceUniqueRedirectsFromCounted);
             for (auto& domain : subresourceUniqueRedirectsFromCounted.values())
                 subresourceUniqueRedirectsFrom.add(domain);
         }
@@ -298,11 +298,11 @@ bool ResourceLoadStatistics::decode(KeyedDecoder& decoder, unsigned modelVersion
 
 
     // Prevalent Resource
-    if (!decoder.decodeBool("isPrevalentResource", isPrevalentResource))
+    if (!decoder.decodeBool("isPrevalentResource"_s, isPrevalentResource))
         return false;
 
     if (modelVersion >= 12) {
-        if (!decoder.decodeBool("isVeryPrevalentResource", isVeryPrevalentResource))
+        if (!decoder.decodeBool("isVeryPrevalentResource"_s, isVeryPrevalentResource))
             return false;
     }
 
@@ -312,129 +312,126 @@ bool ResourceLoadStatistics::decode(KeyedDecoder& decoder, unsigned modelVersion
         isVeryPrevalentResource = false;
     }
 
-    if (!decoder.decodeUInt32("dataRecordsRemoved", dataRecordsRemoved))
+    if (!decoder.decodeUInt32("dataRecordsRemoved"_s, dataRecordsRemoved))
         return false;
 
     double mostRecentUserInteractionTimeAsDouble;
-    if (!decoder.decodeDouble("mostRecentUserInteraction", mostRecentUserInteractionTimeAsDouble))
+    if (!decoder.decodeDouble("mostRecentUserInteraction"_s, mostRecentUserInteractionTimeAsDouble))
         return false;
     mostRecentUserInteractionTime = WallTime::fromRawSeconds(mostRecentUserInteractionTimeAsDouble);
 
-    if (!decoder.decodeBool("grandfathered", grandfathered))
+    if (!decoder.decodeBool("grandfathered"_s, grandfathered))
         return false;
 
     double lastSeenTimeAsDouble;
-    if (!decoder.decodeDouble("lastSeen", lastSeenTimeAsDouble))
+    if (!decoder.decodeDouble("lastSeen"_s, lastSeenTimeAsDouble))
         return false;
     lastSeen = WallTime::fromRawSeconds(lastSeenTimeAsDouble);
 
     if (modelVersion >= 11) {
-        if (!decoder.decodeUInt32("timesAccessedAsFirstPartyDueToUserInteraction", timesAccessedAsFirstPartyDueToUserInteraction))
+        if (!decoder.decodeUInt32("timesAccessedAsFirstPartyDueToUserInteraction"_s, timesAccessedAsFirstPartyDueToUserInteraction))
             timesAccessedAsFirstPartyDueToUserInteraction = 0;
-        if (!decoder.decodeUInt32("timesAccessedAsFirstPartyDueToStorageAccessAPI", timesAccessedAsFirstPartyDueToStorageAccessAPI))
+        if (!decoder.decodeUInt32("timesAccessedAsFirstPartyDueToStorageAccessAPI"_s, timesAccessedAsFirstPartyDueToStorageAccessAPI))
             timesAccessedAsFirstPartyDueToStorageAccessAPI = 0;
     }
 
 #if ENABLE(WEB_API_STATISTICS)
     if (modelVersion >= 13) {
-        decodeFontHashSet(decoder, "fontsFailedToLoad", fontsFailedToLoad);
-        decodeFontHashSet(decoder, "fontsSuccessfullyLoaded", fontsSuccessfullyLoaded);
-        decodeHashSet(decoder, "topFrameRegistrableDomainsWhichAccessedWebAPIs", "domain", topFrameRegistrableDomainsWhichAccessedWebAPIs);
-        decodeCanvasActivityRecord(decoder, "canvasActivityRecord", canvasActivityRecord);
-        decodeOptionSet(decoder, "navigatorFunctionsAccessedBitMask", navigatorFunctionsAccessed);
-        decodeOptionSet(decoder, "screenFunctionsAccessedBitMask", screenFunctionsAccessed);
+        decodeFontHashSet(decoder, "fontsFailedToLoad"_s, fontsFailedToLoad);
+        decodeFontHashSet(decoder, "fontsSuccessfullyLoaded"_s, fontsSuccessfullyLoaded);
+        decodeHashSet(decoder, "topFrameRegistrableDomainsWhichAccessedWebAPIs"_s, "domain"_s, topFrameRegistrableDomainsWhichAccessedWebAPIs);
+        decodeCanvasActivityRecord(decoder, "canvasActivityRecord"_s, canvasActivityRecord);
+        decodeOptionSet(decoder, "navigatorFunctionsAccessedBitMask"_s, navigatorFunctionsAccessed);
+        decodeOptionSet(decoder, "screenFunctionsAccessedBitMask"_s, screenFunctionsAccessed);
     }
 #endif
 
     return true;
 }
 
-static void appendBoolean(StringBuilder& builder, const String& label, bool flag)
+static void appendBoolean(StringBuilder& builder, ASCIILiteral label, bool flag)
 {
-    builder.append("    ", label, ": ", flag ? "Yes" : "No");
+    builder.append("    "_s, label, ": "_s, flag ? "Yes"_s : "No"_s);
 }
 
 static void appendHashSet(StringBuilder& builder, const String& label, const HashSet<RegistrableDomain>& hashSet)
 {
     if (hashSet.isEmpty())
         return;
-    
-    builder.append("    ", label, ":\n");
+
+    builder.append("    "_s, label, ":\n"_s);
     for (auto& entry : hashSet)
-        builder.append("        ", entry.string(), '\n');
+        builder.append("        "_s, entry.string(), '\n');
 }
 
 #if ENABLE(WEB_API_STATISTICS)
-static void appendHashSet(StringBuilder& builder, const String& label, const HashSet<String>& hashSet)
+template<typename HashSetType>
+static void appendHashSet(StringBuilder& builder, const String& label, const HashSetType& hashSet)
 {
     if (hashSet.isEmpty())
         return;
-    
-    builder.append("    ", label, ":\n");
+
+    builder.append("    "_s, label, ":\n"_s);
     for (auto& entry : hashSet)
-        builder.append("        ", entry, '\n');
+        builder.append("        "_s, entry, '\n');
 }
 
-static ASCIILiteral navigatorAPIEnumToString(ResourceLoadStatistics::NavigatorAPI navigatorEnum)
+static ASCIILiteral navigatorAPIEnumToString(NavigatorAPIsAccessed navigatorEnum)
 {
     switch (navigatorEnum) {
-    case ResourceLoadStatistics::NavigatorAPI::JavaEnabled:
-        return "javaEnabled"_s;
-    case ResourceLoadStatistics::NavigatorAPI::MimeTypes:
+    case NavigatorAPIsAccessed::MimeTypes:
         return "mimeTypes"_s;
-    case ResourceLoadStatistics::NavigatorAPI::CookieEnabled:
+    case NavigatorAPIsAccessed::CookieEnabled:
         return "cookieEnabled"_s;
-    case ResourceLoadStatistics::NavigatorAPI::Plugins:
+    case NavigatorAPIsAccessed::Plugins:
         return "plugins"_s;
-    case ResourceLoadStatistics::NavigatorAPI::UserAgent:
+    case NavigatorAPIsAccessed::UserAgent:
         return "userAgent"_s;
-    case ResourceLoadStatistics::NavigatorAPI::AppVersion:
+    case NavigatorAPIsAccessed::AppVersion:
         return "appVersion"_s;
     }
     ASSERT_NOT_REACHED();
     return "Invalid navigator API"_s;
 }
 
-static ASCIILiteral screenAPIEnumToString(ResourceLoadStatistics::ScreenAPI screenEnum)
+static ASCIILiteral screenAPIEnumToString(ScreenAPIsAccessed screenEnum)
 {
     switch (screenEnum) {
-    case ResourceLoadStatistics::ScreenAPI::Height:
+    case ScreenAPIsAccessed::Height:
         return "height"_s;
-    case ResourceLoadStatistics::ScreenAPI::Width:
+    case ScreenAPIsAccessed::Width:
         return "width"_s;
-    case ResourceLoadStatistics::ScreenAPI::ColorDepth:
+    case ScreenAPIsAccessed::ColorDepth:
         return "colorDepth"_s;
-    case ResourceLoadStatistics::ScreenAPI::PixelDepth:
-        return "pixelDepth"_s;
-    case ResourceLoadStatistics::ScreenAPI::AvailLeft:
+    case ScreenAPIsAccessed::AvailLeft:
         return "availLeft"_s;
-    case ResourceLoadStatistics::ScreenAPI::AvailTop:
+    case ScreenAPIsAccessed::AvailTop:
         return "availTop"_s;
-    case ResourceLoadStatistics::ScreenAPI::AvailHeight:
+    case ScreenAPIsAccessed::AvailHeight:
         return "availHeight"_s;
-    case ResourceLoadStatistics::ScreenAPI::AvailWidth:
+    case ScreenAPIsAccessed::AvailWidth:
         return "availWidth"_s;
     }
     ASSERT_NOT_REACHED();
     return "Invalid screen API"_s;
 }
-    
-static void appendNavigatorAPIOptionSet(StringBuilder& builder, const OptionSet<ResourceLoadStatistics::NavigatorAPI>& optionSet)
+
+static void appendNavigatorAPIOptionSet(StringBuilder& builder, const OptionSet<NavigatorAPIsAccessed>& optionSet)
 {
     if (optionSet.isEmpty())
         return;
-    builder.append("    navigatorFunctionsAccessed:\n");
+    builder.append("    navigatorFunctionsAccessed:\n"_s);
     for (auto navigatorAPI : optionSet)
-        builder.append("        ", navigatorAPIEnumToString(navigatorAPI), '\n');
+        builder.append("        "_s, navigatorAPIEnumToString(navigatorAPI), '\n');
 }
-    
-static void appendScreenAPIOptionSet(StringBuilder& builder, const OptionSet<ResourceLoadStatistics::ScreenAPI>& optionSet)
+
+static void appendScreenAPIOptionSet(StringBuilder& builder, const OptionSet<ScreenAPIsAccessed>& optionSet)
 {
     if (optionSet.isEmpty())
         return;
-    builder.append("    screenFunctionsAccessed:\n");
+    builder.append("    screenFunctionsAccessed:\n"_s);
     for (auto screenAPI : optionSet) {
-        builder.append("        ", screenAPIEnumToString(screenAPI), '\n');
+        builder.append("        "_s, screenAPIEnumToString(screenAPI), '\n');
     }
 }
 #endif
@@ -447,53 +444,48 @@ static bool hasHadRecentUserInteraction(Seconds interactionTimeSeconds)
 String ResourceLoadStatistics::toString() const
 {
     StringBuilder builder;
-    builder.append("Registrable domain: ", registrableDomain.string(), '\n');
+    builder.append("Registrable domain: "_s, registrableDomain.string(), '\n');
 
     // User interaction
-    appendBoolean(builder, "hadUserInteraction", hadUserInteraction);
-    builder.append('\n');
-    builder.append("    mostRecentUserInteraction: ", hasHadRecentUserInteraction(mostRecentUserInteractionTime.secondsSinceEpoch()) ? "within 24 hours" : "-1");
-    builder.append('\n');
-    appendBoolean(builder, "grandfathered", grandfathered);
+    appendBoolean(builder, "hadUserInteraction"_s, hadUserInteraction);
+    builder.append("\n    mostRecentUserInteraction: "_s, hasHadRecentUserInteraction(mostRecentUserInteractionTime.secondsSinceEpoch()) ? "within 24 hours\n"_s : "-1\n"_s);
+    appendBoolean(builder, "grandfathered"_s, grandfathered);
     builder.append('\n');
 
     // Storage access
-    appendHashSet(builder, "storageAccessUnderTopFrameDomains", storageAccessUnderTopFrameDomains);
+    appendHashSet(builder, "storageAccessUnderTopFrameDomains"_s, storageAccessUnderTopFrameDomains);
 
     // Top frame stats
-    appendHashSet(builder, "topFrameUniqueRedirectsTo", topFrameUniqueRedirectsTo);
-    appendHashSet(builder, "topFrameUniqueRedirectsFrom", topFrameUniqueRedirectsFrom);
-    appendHashSet(builder, "topFrameLinkDecorationsFrom", topFrameLinkDecorationsFrom);
-    appendBoolean(builder, "gotLinkDecorationFromPrevalentResource", gotLinkDecorationFromPrevalentResource);
+    appendHashSet(builder, "topFrameUniqueRedirectsTo"_s, topFrameUniqueRedirectsTo);
+    appendHashSet(builder, "topFrameUniqueRedirectsFrom"_s, topFrameUniqueRedirectsFrom);
+    appendHashSet(builder, "topFrameLinkDecorationsFrom"_s, topFrameLinkDecorationsFrom);
+    appendBoolean(builder, "gotLinkDecorationFromPrevalentResource"_s, gotLinkDecorationFromPrevalentResource);
     builder.append('\n');
-    appendHashSet(builder, "topFrameLoadedThirdPartyScripts", topFrameLoadedThirdPartyScripts);
+    appendHashSet(builder, "topFrameLoadedThirdPartyScripts"_s, topFrameLoadedThirdPartyScripts);
 
     // Subframe stats
-    appendHashSet(builder, "subframeUnderTopFrameDomains", subframeUnderTopFrameDomains);
-    
+    appendHashSet(builder, "subframeUnderTopFrameDomains"_s, subframeUnderTopFrameDomains);
+
     // Subresource stats
-    appendHashSet(builder, "subresourceUnderTopFrameDomains", subresourceUnderTopFrameDomains);
-    appendHashSet(builder, "subresourceUniqueRedirectsTo", subresourceUniqueRedirectsTo);
-    appendHashSet(builder, "subresourceUniqueRedirectsFrom", subresourceUniqueRedirectsFrom);
+    appendHashSet(builder, "subresourceUnderTopFrameDomains"_s, subresourceUnderTopFrameDomains);
+    appendHashSet(builder, "subresourceUniqueRedirectsTo"_s, subresourceUniqueRedirectsTo);
+    appendHashSet(builder, "subresourceUniqueRedirectsFrom"_s, subresourceUniqueRedirectsFrom);
 
     // Prevalent Resource
-    appendBoolean(builder, "isPrevalentResource", isPrevalentResource);
-    builder.append('\n');
-    appendBoolean(builder, "isVeryPrevalentResource", isVeryPrevalentResource);
-    builder.append('\n');
-    builder.append("    dataRecordsRemoved: ", dataRecordsRemoved);
+    appendBoolean(builder, "isPrevalentResource"_s, isPrevalentResource);
     builder.append('\n');
+    appendBoolean(builder, "isVeryPrevalentResource"_s, isVeryPrevalentResource);
+    builder.append("\n    dataRecordsRemoved: "_s, dataRecordsRemoved, '\n');
 
 #if ENABLE(WEB_API_STATISTICS)
-    appendHashSet(builder, "fontsFailedToLoad", fontsFailedToLoad);
-    appendHashSet(builder, "fontsSuccessfullyLoaded", fontsSuccessfullyLoaded);
-    appendHashSet(builder, "topFrameRegistrableDomainsWhichAccessedWebAPIs", topFrameRegistrableDomainsWhichAccessedWebAPIs);
+    appendHashSet(builder, "fontsFailedToLoad"_s, fontsFailedToLoad);
+    appendHashSet(builder, "fontsSuccessfullyLoaded"_s, fontsSuccessfullyLoaded);
+    appendHashSet(builder, "topFrameRegistrableDomainsWhichAccessedWebAPIs"_s, topFrameRegistrableDomainsWhichAccessedWebAPIs);
     appendNavigatorAPIOptionSet(builder, navigatorFunctionsAccessed);
     appendScreenAPIOptionSet(builder, screenFunctionsAccessed);
-    appendHashSet(builder, "canvasTextWritten", canvasActivityRecord.textWritten);
-    appendBoolean(builder, "canvasReadData", canvasActivityRecord.wasDataRead);
-    builder.append('\n');
-    builder.append('\n');
+    appendHashSet(builder, "canvasTextWritten"_s, canvasActivityRecord.textWritten);
+    appendBoolean(builder, "canvasReadData"_s, canvasActivityRecord.wasDataRead);
+    builder.append("\n\n"_s);
 #endif
 
     return builder.toString();
@@ -519,7 +511,7 @@ void ResourceLoadStatistics::merge(const ResourceLoadStatistics& other)
 
     if (lastSeen < other.lastSeen)
         lastSeen = other.lastSeen;
-    
+
     if (!other.hadUserInteraction) {
         // If user interaction has been reset do so here too.
         // Else, do nothing.
@@ -546,7 +538,7 @@ void ResourceLoadStatistics::merge(const ResourceLoadStatistics& other)
 
     // Subframe stats
     mergeHashSet(subframeUnderTopFrameDomains, other.subframeUnderTopFrameDomains);
-    
+
     // Subresource stats
     mergeHashSet(subresourceUnderTopFrameDomains, other.subresourceUnderTopFrameDomains);
     mergeHashSet(subresourceUniqueRedirectsTo, other.subresourceUniqueRedirectsTo);
@@ -556,7 +548,7 @@ void ResourceLoadStatistics::merge(const ResourceLoadStatistics& other)
     isPrevalentResource |= other.isPrevalentResource;
     isVeryPrevalentResource |= other.isVeryPrevalentResource;
     dataRecordsRemoved = std::max(dataRecordsRemoved, other.dataRecordsRemoved);
-    
+
 #if ENABLE(WEB_API_STATISTICS)
     mergeHashSet(fontsFailedToLoad, other.fontsFailedToLoad);
     mergeHashSet(fontsSuccessfullyLoaded, other.fontsSuccessfullyLoaded);
