@@ -20,21 +20,35 @@
 #pragma once
 
 #if USE(GSTREAMER)
-#include "FloatSize.h"
 #include "GRefPtrGStreamer.h"
-#include "GUniquePtrGStreamer.h"
-#include "PlatformVideoColorSpace.h"
+
+#include <gst/audio/audio-buffer.h>
+#include <gst/audio/audio-info.h>
 #include <gst/gst.h>
 #include <gst/video/video-format.h>
 #include <gst/video/video-info.h>
-#include <wtf/MediaTime.h>
+#include <wtf/Logger.h>
+#include <wtf/StdLibExtras.h>
+#include <wtf/TZoneMalloc.h>
 #include <wtf/ThreadSafeRefCounted.h>
 
+namespace WTF {
+class MediaTime;
+}
+
 namespace WebCore {
 
+class FloatSize;
 class IntSize;
 class SharedBuffer;
 
+struct PlatformVideoColorSpace;
+
+using TrackID = uint64_t;
+
+template<typename MappedArg>
+using TrackIDHashMap = HashMap<TrackID, MappedArg, WTF::IntHash<TrackID>, WTF::UnsignedWithZeroKeyHashTraits<TrackID>>;
+
 inline bool webkitGstCheckVersion(guint major, guint minor, guint micro)
 {
     guint currentMajor, currentMinor, currentMicro, currentNano;
@@ -56,56 +70,39 @@ inline bool webkitGstCheckVersion(guint major, guint minor, guint micro)
     return true;
 }
 
-// gst_video_format_info_component() is GStreamer 1.18 API, so for older versions we use a local
-// vendored copy of the function.
-#if !GST_CHECK_VERSION(1, 18, 0)
-#define GST_VIDEO_MAX_COMPONENTS 4
-void webkitGstVideoFormatInfoComponent(const GstVideoFormatInfo*, guint, gint components[GST_VIDEO_MAX_COMPONENTS]);
+#define GST_VIDEO_CAPS_TYPE_PREFIX  "video/"_s
+#define GST_AUDIO_CAPS_TYPE_PREFIX  "audio/"_s
+#define GST_TEXT_CAPS_TYPE_PREFIX   "text/"_s
 
-#define gst_video_format_info_component webkitGstVideoFormatInfoComponent
-#endif
-
-#define GST_VIDEO_CAPS_TYPE_PREFIX  "video/"
-#define GST_AUDIO_CAPS_TYPE_PREFIX  "audio/"
-#define GST_TEXT_CAPS_TYPE_PREFIX   "text/"
-
-GstPad* webkitGstGhostPadFromStaticTemplate(GstStaticPadTemplate*, const gchar* name, GstPad* target);
+WARN_UNUSED_RETURN GstPad* webkitGstGhostPadFromStaticTemplate(GstStaticPadTemplate*, ASCIILiteral name, GstPad* target);
 #if ENABLE(VIDEO)
-bool getVideoSizeAndFormatFromCaps(const GstCaps*, WebCore::IntSize&, GstVideoFormat&, int& pixelAspectRatioNumerator, int& pixelAspectRatioDenominator, int& stride);
+bool getVideoSizeAndFormatFromCaps(const GstCaps*, WebCore::IntSize&, GstVideoFormat&, int& pixelAspectRatioNumerator, int& pixelAspectRatioDenominator, int& stride, double& frameRate, PlatformVideoColorSpace&);
 std::optional<FloatSize> getVideoResolutionFromCaps(const GstCaps*);
 bool getSampleVideoInfo(GstSample*, GstVideoInfo&);
 #endif
-const char* capsMediaType(const GstCaps*);
-bool doCapsHaveType(const GstCaps*, const char*);
+StringView capsMediaType(const GstCaps*);
+std::optional<TrackID> getStreamIdFromPad(const GRefPtr<GstPad>&);
+std::optional<TrackID> getStreamIdFromStream(const GRefPtr<GstStream>&);
+std::optional<TrackID> parseStreamId(StringView stringId);
+bool doCapsHaveType(const GstCaps*, ASCIILiteral);
 bool areEncryptedCaps(const GstCaps*);
 Vector<String> extractGStreamerOptionsFromCommandLine();
 void setGStreamerOptionsFromUIProcess(Vector<String>&&);
 bool ensureGStreamerInitialized();
 void registerWebKitGStreamerElements();
 void registerWebKitGStreamerVideoEncoder();
-unsigned getGstPlayFlag(const char* nick);
-uint64_t toGstUnsigned64Time(const MediaTime&);
-#if ENABLE(THUNDER)
-bool isThunderRanked();
-#endif
+void deinitializeGStreamer();
 
-inline GstClockTime toGstClockTime(const MediaTime& mediaTime)
+unsigned getGstPlayFlag(const char* nick);
+uint64_t toGstUnsigned64Time(const WTF::MediaTime&);
+
+inline GstClockTime toGstClockTime(const WTF::MediaTime& mediaTime)
 {
     return static_cast<GstClockTime>(toGstUnsigned64Time(mediaTime));
 }
 
-inline GstClockTime toGstClockTime(const Seconds& seconds)
-{
-    return toGstClockTime(MediaTime::createWithDouble(seconds.seconds()));
-}
-
-inline MediaTime fromGstClockTime(GstClockTime time)
-{
-    if (!GST_CLOCK_TIME_IS_VALID(time))
-        return MediaTime::invalidTime();
-
-    return MediaTime(GST_TIME_AS_USECONDS(time), G_USEC_PER_SEC);
-}
+GstClockTime toGstClockTime(const Seconds&);
+WTF::MediaTime fromGstClockTime(GstClockTime);
 
 template<typename MapType, gboolean(mapFunction)(GstBuffer*, MapType*, GstMapFlags), void(unmapFunction)(GstBuffer*, MapType*)>
 class GstBufferMapper {
@@ -154,6 +151,8 @@ public:
     bool isValid() const { return m_isValid; }
     uint8_t* data() { RELEASE_ASSERT(m_isValid); return static_cast<uint8_t*>(m_info.data); }
     const uint8_t* data() const { RELEASE_ASSERT(m_isValid); return static_cast<uint8_t*>(m_info.data); }
+    template<typename T> std::span<T> mutableSpan() { return unsafeMakeSpan(reinterpret_cast<T*>(data()), size() / sizeof(T)); }
+    template<typename T> std::span<const T> span() const { return unsafeMakeSpan(reinterpret_cast<const T*>(data()), size() / sizeof(T)); }
     size_t size() const { ASSERT(m_isValid); return m_isValid ? static_cast<size_t>(m_info.size) : 0; }
     MapType* mappedData() const  { ASSERT(m_isValid); return m_isValid ? const_cast<MapType*>(&m_info) : nullptr; }
     Vector<uint8_t> createVector() const;
@@ -176,8 +175,6 @@ private:
         return a.isValid() && GST_IS_BUFFER(b) && a.size() == gst_buffer_get_size(nonConstB) && !gst_buffer_memcmp(nonConstB, 0, a.data(), a.size());
     }
 
-    friend bool operator==(const GstBuffer* a, const GstBufferMapper& b) { return operator==(b, a); }
-
     GstBuffer* m_buffer { nullptr };
     MapType m_info;
     bool m_isValid { false };
@@ -191,33 +188,14 @@ using GstMappedBuffer = GstBufferMapper<GstMapInfo, gst_buffer_map, gst_buffer_u
 class GstMappedOwnedBuffer : public GstMappedBuffer, public ThreadSafeRefCounted<GstMappedOwnedBuffer> {
 
 public:
-    static RefPtr<GstMappedOwnedBuffer> create(GRefPtr<GstBuffer>&& buffer)
-    {
-        auto* mappedBuffer = new GstMappedOwnedBuffer(WTFMove(buffer));
-        if (!mappedBuffer->isValid()) {
-            delete mappedBuffer;
-            return nullptr;
-        }
-
-        return adoptRef(mappedBuffer);
-    }
-
-    static RefPtr<GstMappedOwnedBuffer> create(const GRefPtr<GstBuffer>& buffer)
-    {
-        return create(GRefPtr(buffer));
-    }
+    static RefPtr<GstMappedOwnedBuffer> create(GRefPtr<GstBuffer>&&);
+    static RefPtr<GstMappedOwnedBuffer> create(const GRefPtr<GstBuffer>&);
 
     // This GstBuffer is [ transfer none ], meaning the reference
     // count is increased during the life of this object.
-    static RefPtr<GstMappedOwnedBuffer> create(GstBuffer* buffer)
-    {
-        return create(GRefPtr(buffer));
-    }
+    static RefPtr<GstMappedOwnedBuffer> create(GstBuffer*);
 
-    virtual ~GstMappedOwnedBuffer()
-    {
-        unmapEarly();
-    }
+    virtual ~GstMappedOwnedBuffer();
 
     Ref<SharedBuffer> createSharedBuffer();
 
@@ -230,94 +208,63 @@ private:
 };
 
 class GstMappedFrame {
+    WTF_MAKE_TZONE_ALLOCATED(GstMappedFrame);
     WTF_MAKE_NONCOPYABLE(GstMappedFrame);
 public:
+    GstMappedFrame(GstBuffer*, const GstVideoInfo*, GstMapFlags);
+    GstMappedFrame(const GRefPtr<GstSample>&, GstMapFlags);
 
-    GstMappedFrame(GstBuffer* buffer, GstVideoInfo info, GstMapFlags flags)
-    {
-        m_isValid = gst_video_frame_map(&m_frame, &info, buffer, flags);
-    }
+    ~GstMappedFrame();
 
-    GstMappedFrame(GRefPtr<GstSample> sample, GstMapFlags flags)
-    {
-        GstVideoInfo info;
+    GstVideoFrame* get();
 
-        if (!gst_video_info_from_caps(&info, gst_sample_get_caps(sample.get()))) {
-            m_isValid = false;
-            return;
-        }
+    uint8_t* componentData(int) const;
+    int componentStride(int) const;
+    int componentWidth(int) const;
 
-        m_isValid = gst_video_frame_map(&m_frame, &info, gst_sample_get_buffer(sample.get()), flags);
-    }
+    GstVideoInfo* info();
 
-    GstVideoFrame* get()
-    {
-        if (!m_isValid) {
-            GST_INFO("Invalid frame, returning NULL");
+    int width() const;
+    int height() const;
 
-            return nullptr;
-        }
+    int format() const;
+    void* planeData(uint32_t) const;
+    int planeStride(uint32_t) const;
 
-        return &m_frame;
-    }
+    bool isValid() const { return m_frame.buffer; }
+    explicit operator bool() const { return m_frame.buffer; }
+    bool operator!() const { return !m_frame.buffer; }
 
-    uint8_t* ComponentData(int comp)
-    {
-        return GST_VIDEO_FRAME_COMP_DATA(&m_frame, comp);
-    }
+private:
+    GstVideoFrame m_frame;
+};
 
-    int ComponentStride(int stride)
-    {
-        return GST_VIDEO_FRAME_COMP_STRIDE(&m_frame, stride);
-    }
+class GstMappedAudioBuffer {
+    WTF_MAKE_TZONE_ALLOCATED(GstMappedAudioBuffer);
+    WTF_MAKE_NONCOPYABLE(GstMappedAudioBuffer);
+public:
+    GstMappedAudioBuffer(GstBuffer*, GstAudioInfo, GstMapFlags);
+    GstMappedAudioBuffer(const GRefPtr<GstSample>&, GstMapFlags);
+    ~GstMappedAudioBuffer();
 
-    GstVideoInfo* info()
-    {
-        if (!m_isValid) {
-            GST_INFO("Invalid frame, returning NULL");
+    GstAudioBuffer* get();
+    GstAudioInfo* info();
 
-            return nullptr;
-        }
-
-        return &m_frame.info;
-    }
-
-    int width()
-    {
-        return m_isValid ? GST_VIDEO_FRAME_WIDTH(&m_frame) : -1;
-    }
-
-    int height()
-    {
-        return m_isValid ? GST_VIDEO_FRAME_HEIGHT(&m_frame) : -1;
-    }
-
-    int format()
-    {
-        return m_isValid ? GST_VIDEO_FRAME_FORMAT(&m_frame) : GST_VIDEO_FORMAT_UNKNOWN;
-    }
-
-    ~GstMappedFrame()
-    {
-        if (m_isValid)
-            gst_video_frame_unmap(&m_frame);
-        m_isValid = false;
-    }
+    template<typename T> Vector<std::span<T>> samples(size_t offset) const;
 
     explicit operator bool() const { return m_isValid; }
 
 private:
-    GstVideoFrame m_frame;
+    GstAudioBuffer m_buffer;
     bool m_isValid { false };
 };
 
-
 void connectSimpleBusMessageCallback(GstElement*, Function<void(GstMessage*)>&& = [](GstMessage*) { });
 void disconnectSimpleBusMessageCallback(GstElement*);
 
 enum class GstVideoDecoderPlatform { ImxVPU, Video4Linux, OpenMAX };
 
-bool isGStreamerPluginAvailable(const char* name);
+bool isGStreamerPluginAvailable(ASCIILiteral name);
 bool gstElementFactoryEquals(GstElement*, ASCIILiteral name);
 
 GstElement* createAutoAudioSink(const String& role);
@@ -330,22 +277,93 @@ bool webkitGstSetElementStateSynchronously(GstElement*, GstState, Function<bool(
 GstBuffer* gstBufferNewWrappedFast(void* data, size_t length);
 
 // These functions should be used for elements not provided by WebKit itself and not provided by GStreamer -core.
-GstElement* makeGStreamerElement(const char* factoryName, const char* name);
-GstElement* makeGStreamerBin(const char* description, bool ghostUnlinkedPads);
+GstElement* makeGStreamerElement(ASCIILiteral factoryName, const String& name = emptyString());
+
+template<typename T>
+std::optional<T> gstStructureGet(const GstStructure*, ASCIILiteral key);
+template<typename T>
+std::optional<T> gstStructureGet(const GstStructure*, StringView key);
+
+StringView gstStructureGetString(const GstStructure*, ASCIILiteral key);
+StringView gstStructureGetString(const GstStructure*, StringView key);
+
+StringView gstStructureGetName(const GstStructure*);
+
+template<typename T>
+Vector<T> gstStructureGetArray(const GstStructure*, ASCIILiteral key);
+
+template<typename T>
+Vector<T> gstStructureGetList(const GstStructure*, ASCIILiteral key);
 
 String gstStructureToJSONString(const GstStructure*);
 
-// gst_element_get_current_running_time() is GStreamer 1.18 API, so for older versions we use a local
-// vendored copy of the function.
-#if !GST_CHECK_VERSION(1, 18, 0)
-GstClockTime webkitGstElementGetCurrentRunningTime(GstElement*);
-#define gst_element_get_current_running_time webkitGstElementGetCurrentRunningTime
-#endif
+GstClockTime webkitGstInitTime();
 
 PlatformVideoColorSpace videoColorSpaceFromCaps(const GstCaps*);
 PlatformVideoColorSpace videoColorSpaceFromInfo(const GstVideoInfo&);
 void fillVideoInfoColorimetryFromColorSpace(GstVideoInfo*, const PlatformVideoColorSpace&);
 
+void configureAudioDecoderForHarnessing(const GRefPtr<GstElement>&);
+void configureVideoDecoderForHarnessing(const GRefPtr<GstElement>&);
+
+void configureMediaStreamAudioDecoder(GstElement*);
+
+void configureMediaStreamVideoDecoder(GstElement*);
+void configureVideoRTPDepayloader(GstElement*);
+
+bool gstObjectHasProperty(GstObject*, ASCIILiteral name);
+bool gstObjectHasProperty(GstElement*, ASCIILiteral name);
+bool gstObjectHasProperty(GstPad*, ASCIILiteral name);
+
+bool gstElementMatchesFactoryAndHasProperty(GstElement*, ASCIILiteral factoryNamePattern, ASCIILiteral propertyName);
+
+GRefPtr<GstBuffer> wrapSpanData(const std::span<const uint8_t>&);
+
+std::optional<unsigned> gstGetAutoplugSelectResult(ASCIILiteral);
+
+void registerActivePipeline(const GRefPtr<GstElement>&);
+void unregisterPipeline(const GRefPtr<GstElement>&);
+
+class WebCoreLogObserver : public Logger::Observer {
+    WTF_MAKE_TZONE_ALLOCATED(WebCoreLogObserver);
+    WTF_MAKE_NONCOPYABLE(WebCoreLogObserver);
+    friend NeverDestroyed<WebCoreLogObserver>;
+public:
+    explicit WebCoreLogObserver() = default;
+    void didLogMessage(const WTFLogChannel&, WTFLogLevel, Vector<JSONLogValue>&&) final;
+
+    virtual GstDebugCategory* debugCategory() const = 0;
+    virtual bool shouldEmitLogMessage(const WTFLogChannel&) const = 0;
+
+    void addWatch(const Logger&);
+    void removeWatch(const Logger&);
+
+private:
+    Atomic<uint64_t> m_totalObservers;
+};
+
+#if GST_CHECK_VERSION(1, 26, 0)
+using GstId = const GstIdStr*;
+#else
+using GstId = GQuark;
+#endif
+
+bool gstStructureForeach(const GstStructure*, Function<bool(GstId, const GValue*)>&&);
+void gstStructureIdSetValue(GstStructure*, GstId, const GValue*);
+bool gstStructureMapInPlace(GstStructure*, Function<bool(GstId, GValue*)>&&);
+StringView gstIdToString(GstId);
+void gstStructureFilterAndMapInPlace(GstStructure*, Function<bool(GstId, GValue*)>&&);
+
+#if USE(GBM)
+WARN_UNUSED_RETURN GRefPtr<GstCaps> buildDMABufCaps();
+#endif
+
+#if USE(GSTREAMER_GL)
+bool setGstElementGLContext(GstElement*, ASCIILiteral contextType);
+#endif
+
+GstStateChangeReturn gstElementLockAndSetState(GstElement*, GstState);
+
 } // namespace WebCore
 
 #ifndef GST_BUFFER_DTS_OR_PTS
@@ -381,10 +399,15 @@ using GstStateLocker = ExternalLocker<void, gstStateLock, gstStateUnlock>;
 template <typename T>
 class GstIteratorAdaptor {
 public:
-    GstIteratorAdaptor(GUniquePtr<GstIterator>&& iter)
-        : m_iter(WTFMove(iter))
+    GstIteratorAdaptor(GstIterator* /* transfer full */ iter)
+        : m_iter(iter)
     { }
 
+    ~GstIteratorAdaptor()
+    {
+        gst_iterator_free(m_iter);
+    }
+
     class iterator {
     public:
         iterator(GstIterator* iter, gboolean done = FALSE)
@@ -419,7 +442,6 @@ public:
         {
             return m_iter == other.m_iter && m_done == other.m_done;
         }
-        bool operator!=(const iterator& other) const { return !(*this == other); }
 
     private:
         GstIterator* m_iter;
@@ -431,18 +453,26 @@ public:
     {
         ASSERT(!m_started);
         m_started = true;
-        iterator iter { m_iter.get() };
+        iterator iter { m_iter };
         return ++iter;
     }
 
     iterator end()
     {
-        return { m_iter.get(), TRUE };
+        return { m_iter, TRUE };
     }
 
 private:
-    GUniquePtr<GstIterator> m_iter;
+    GstIterator* m_iter;
     bool m_started { false };
 };
 
+#if !GST_CHECK_VERSION(1, 20, 0)
+GstBuffer* gst_buffer_new_memdup(gconstpointer data, gsize size);
+#endif
+
+#if !GST_CHECK_VERSION(1, 27, 0)
+void gst_pad_probe_info_set_buffer(GstPadProbeInfo*, GstBuffer*);
+#endif
+
 #endif // USE(GSTREAMER)
