@@ -20,15 +20,17 @@
 #include "config.h"
 
 #include "SVGMissingGlyphElement.h"
+
 #include "SVGNames.h"
-#include <wtf/IsoMallocInlines.h>
+#include "SVGPropertyOwnerRegistry.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(SVGMissingGlyphElement);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(SVGMissingGlyphElement);
 
 inline SVGMissingGlyphElement::SVGMissingGlyphElement(const QualifiedName& tagName, Document& document)
-    : SVGElement(tagName, document)
+    : SVGElement(tagName, document, makeUniqueRef<PropertyRegistry>(*this))
 {
     ASSERT(hasTagName(SVGNames::missing_glyphTag));
 }
