@@ -26,7 +26,8 @@ namespace WebCore {
 
 // SVGAnimateElement implements superset of the functionality.
 class SVGSetElement final : public SVGAnimateElementBase {
-    WTF_MAKE_ISO_ALLOCATED(SVGSetElement);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(SVGSetElement);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(SVGSetElement);
 public:
     static Ref<SVGSetElement> create(const QualifiedName&, Document&);
 
