@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 #include "NetworkConnectionToWebProcess.h"
 #include "NetworkDataTask.h"
 #include <WebCore/FetchIdentifier.h>
-#include <wtf/FileSystem.h>
+#include <wtf/FileHandle.h>
 #include <wtf/FunctionDispatcher.h>
 #include <wtf/TZoneMalloc.h>
 
@@ -69,10 +69,8 @@ private:
     ServiceWorkerDownloadTask(NetworkSession&, NetworkDataTaskClient&, WebSWServerToContextConnection&, WebCore::ServiceWorkerIdentifier, WebCore::SWServerConnectionIdentifier, WebCore::FetchIdentifier, const WebCore::ResourceRequest&, const WebCore::ResourceResponse& response, DownloadID);
     void startListeningForIPC();
 
-    Ref<NetworkProcess> protectedNetworkProcess() const;
-
     // IPC Message
-    void didReceiveData(const IPC::SharedBufferReference&, uint64_t encodedDataLength);
+    void didReceiveData(const IPC::SharedBufferReference&);
     void didReceiveFormData(const IPC::FormDataReference&);
     void didFinish();
     void didFail(WebCore::ResourceError&&);
@@ -99,9 +97,9 @@ private:
     WebCore::SWServerConnectionIdentifier m_serverConnectionIdentifier;
     WebCore::FetchIdentifier m_fetchIdentifier;
     DownloadID m_downloadID;
-    Ref<NetworkProcess> m_networkProcess;
+    const Ref<NetworkProcess> m_networkProcess;
     RefPtr<SandboxExtension> m_sandboxExtension;
-    FileSystem::PlatformFileHandle m_downloadFile { FileSystem::invalidPlatformFileHandle };
+    FileSystem::FileHandle m_downloadFile;
     uint64_t m_downloadBytesWritten { 0 };
     std::optional<uint64_t> m_expectedContentLength;
     State m_state { State::Suspended };
