@@ -28,6 +28,7 @@
 
 #include "Logging.h"
 #include <WebCore/SharedBuffer.h>
+#include <wtf/FileHandle.h>
 #include <wtf/FileSystem.h>
 #include <wtf/RunLoop.h>
 
@@ -54,15 +55,13 @@ void writeToDisk(std::unique_ptr<KeyedEncoder>&& encoder, String&& path)
     if (!rawData)
         return;
 
-    FileSystem::PlatformFileHandle handle = FileSystem::openAndLockFile(path, FileSystem::FileOpenMode::Truncate);
-    if (handle == FileSystem::invalidPlatformFileHandle)
+    auto handle = FileSystem::openFile(path, FileSystem::FileOpenMode::Truncate, FileSystem::FileAccessPermission::All, { FileSystem::FileLockMode::Exclusive });
+    if (!handle)
         return;
 
-    auto writtenBytes = FileSystem::writeToFile(handle, rawData->span());
-    FileSystem::unlockAndCloseFile(handle);
-
-    if (writtenBytes != static_cast<int64_t>(rawData->size()))
-        RELEASE_LOG_ERROR(DiskPersistency, "Disk persistency: We only wrote %d out of %zu bytes to disk", static_cast<unsigned>(writtenBytes), rawData->size());
+    auto writtenBytes = handle.write(rawData->span());
+    if (writtenBytes != rawData->size())
+        RELEASE_LOG_ERROR(DiskPersistency, "Disk persistency: We only wrote %" PRIu64 " out of %zu bytes to disk", *writtenBytes, rawData->size());
 }
 
 } // namespace WebKit
