@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2010-2017 Apple Inc. All rights reserved.
- * Portions Copyright (c) 2010 Motorola Mobility, Inc.  All rights reserved.
+ * Portions Copyright (c) 2010 Motorola Mobility, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -49,10 +49,10 @@
 #include <WebCore/Settings.h>
 
 namespace API {
+class CaptionUserPreferencesTestingModeToken;
 class ContentRuleList;
 class ContentRuleListStore;
-class InternalDebugFeature;
-class ExperimentalFeature;
+class Feature;
 class FrameHandle;
 class FrameInfo;
 class HTTPCookieStore;
@@ -87,7 +87,6 @@ class WebBackForwardList;
 class WebBackForwardListItem;
 class WebColorPickerResultListenerProxy;
 class WebContextMenuListenerProxy;
-class WebCookieManagerProxy;
 class WebCredential;
 class WebFormSubmissionListenerProxy;
 class WebFramePolicyListenerProxy;
@@ -117,13 +116,14 @@ WK_ADD_API_MAPPING(WKAuthenticationDecisionListenerRef, AuthenticationDecisionLi
 WK_ADD_API_MAPPING(WKBackForwardListItemRef, WebBackForwardListItem)
 WK_ADD_API_MAPPING(WKBackForwardListRef, WebBackForwardList)
 WK_ADD_API_MAPPING(WKBundleHitTestResultMediaType, BundleHitTestResultMediaType)
+WK_ADD_API_MAPPING(WKCaptionUserPreferencesTestingModeTokenRef, API::CaptionUserPreferencesTestingModeToken)
 WK_ADD_API_MAPPING(WKColorPickerResultListenerRef, WebColorPickerResultListenerProxy)
 WK_ADD_API_MAPPING(WKContextRef, WebProcessPool)
 WK_ADD_API_MAPPING(WKContextConfigurationRef, API::ProcessPoolConfiguration)
 WK_ADD_API_MAPPING(WKContextMenuListenerRef, WebContextMenuListenerProxy)
-WK_ADD_API_MAPPING(WKCookieManagerRef, WebCookieManagerProxy)
 WK_ADD_API_MAPPING(WKCredentialRef, WebCredential)
 WK_ADD_API_MAPPING(WKDownloadRef, DownloadProxy)
+WK_ADD_API_MAPPING(WKFeatureRef, API::Feature)
 WK_ADD_API_MAPPING(WKFormSubmissionListenerRef, WebFormSubmissionListenerProxy)
 WK_ADD_API_MAPPING(WKFramePolicyListenerRef, WebFramePolicyListenerProxy)
 WK_ADD_API_MAPPING(WKFrameHandleRef, API::FrameHandle)
@@ -238,16 +238,22 @@ inline WKProcessTerminationReason toAPI(ProcessTerminationReason reason)
         return kWKProcessTerminationReasonExceededMemoryLimit;
     case ProcessTerminationReason::ExceededCPULimit:
         return kWKProcessTerminationReasonExceededCPULimit;
+    case ProcessTerminationReason::IdleExit:
     case ProcessTerminationReason::NavigationSwap:
         // We probably shouldn't bother coming up with a new C-API type for process-swapping.
         // "Requested by client" seems like the best match for existing types.
-        FALLTHROUGH;
+        [[fallthrough]];
     case ProcessTerminationReason::RequestedByClient:
         return kWKProcessTerminationReasonRequestedByClient;
     case ProcessTerminationReason::ExceededProcessCountLimit:
+    case ProcessTerminationReason::Unresponsive:
     case ProcessTerminationReason::RequestedByNetworkProcess:
     case ProcessTerminationReason::RequestedByGPUProcess:
+    case ProcessTerminationReason::RequestedByModelProcess:
     case ProcessTerminationReason::Crash:
+    case ProcessTerminationReason::GPUProcessCrashedTooManyTimes:
+    case ProcessTerminationReason::ModelProcessCrashedTooManyTimes:
+    case ProcessTerminationReason::NonMainFrameWebContentProcessCrash:
         return kWKProcessTerminationReasonCrash;
     }
 
@@ -345,13 +351,13 @@ inline WebCore::CredentialPersistence toCredentialPersistence(WKCredentialPersis
 {
     switch (type) {
     case kWKCredentialPersistenceNone:
-        return WebCore::CredentialPersistenceNone;
+        return WebCore::CredentialPersistence::None;
     case kWKCredentialPersistenceForSession:
-        return WebCore::CredentialPersistenceForSession;
+        return WebCore::CredentialPersistence::ForSession;
     case kWKCredentialPersistencePermanent:
-        return WebCore::CredentialPersistencePermanent;
+        return WebCore::CredentialPersistence::Permanent;
     default:
-        return WebCore::CredentialPersistenceNone;
+        return WebCore::CredentialPersistence::None;
     }
 }
 
@@ -419,36 +425,6 @@ inline WKStorageBlockingPolicy toAPI(WebCore::StorageBlockingPolicy policy)
     return kWKAllowAllStorage;
 }
 
-inline WebCore::WebGLLoadPolicy toWebGLLoadPolicy(WKWebGLLoadPolicy webGLLoadPolicy)
-{
-    switch (webGLLoadPolicy) {
-    case kWKWebGLLoadPolicyLoadNormally:
-        return WebCore::WebGLLoadPolicy::WebGLAllowCreation;
-    case kWKWebGLLoadPolicyBlocked:
-        return WebCore::WebGLLoadPolicy::WebGLBlockCreation;
-    case kWKWebGLLoadPolicyPending:
-        return WebCore::WebGLLoadPolicy::WebGLPendingCreation;
-    }
-    
-    ASSERT_NOT_REACHED();
-    return WebCore::WebGLLoadPolicy::WebGLAllowCreation;
-}
-
-inline WKWebGLLoadPolicy toAPI(WebCore::WebGLLoadPolicy webGLLoadPolicy)
-{
-    switch (webGLLoadPolicy) {
-    case WebCore::WebGLLoadPolicy::WebGLAllowCreation:
-        return kWKWebGLLoadPolicyLoadNormally;
-    case WebCore::WebGLLoadPolicy::WebGLBlockCreation:
-        return kWKWebGLLoadPolicyBlocked;
-    case WebCore::WebGLLoadPolicy::WebGLPendingCreation:
-        return kWKWebGLLoadPolicyPending;
-    }
-
-    ASSERT_NOT_REACHED();
-    return kWKWebGLLoadPolicyLoadNormally;
-}
-
 } // namespace WebKit
 
 #if defined(BUILDING_GTK__)
