@@ -401,7 +401,7 @@ static std::optional<uint32_t> buildDamageBlob(WPEDisplayDRM* display, const Vec
 
     uint32_t blobID;
     int fd = gbm_device_get_fd(wpe_display_drm_get_device(display));
-    auto result = drmModeCreatePropertyBlob(fd, damageRects.data(), damageRects.sizeInBytes(), &blobID);
+    auto result = drmModeCreatePropertyBlob(fd, damageRects.span().data(), damageRects.sizeInBytes(), &blobID);
     if (result < 0) {
         g_set_error(error, WPE_VIEW_ERROR, WPE_VIEW_ERROR_RENDER_FAILED, "Failed to render buffer: failed to crate damage blob: %s", safeStrerror(-result).data());
         return 0;
@@ -498,7 +498,7 @@ static void wpeViewDRMScheduleCursorUpdate(WPEViewDRM* view)
         return;
 
     if (!priv->cursorUpdateTimer) {
-        priv->cursorUpdateTimer = makeUnique<RunLoop::Timer>(RunLoop::current(), [view] {
+        priv->cursorUpdateTimer = makeUnique<RunLoop::Timer>(RunLoop::currentSingleton(), "_WPEViewDRMPrivate::cursorUpdateTimer"_s, [view] {
             if (wpeViewDRMRequestUpdate(view, nullptr))
                 view->priv->updateFlags.add(UpdateFlags::CursorUpdateRequested);
         });
