@@ -26,11 +26,15 @@
 #include "config.h"
 #include "WPEToplevelWayland.h"
 
+#include "GRefPtrWPE.h"
 #include "WPEBufferDMABufFormats.h"
 #include "WPEDisplayWaylandPrivate.h"
 #include "WPEToplevelWaylandPrivate.h"
 #include "linux-dmabuf-unstable-v1-client-protocol.h"
 #include "xdg-shell-client-protocol.h"
+#if USE(XDG_DECORATION_UNSTABLE_V1)
+#include "xdg-decoration-unstable-v1-client-protocol.h"
+#endif
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <wtf/Vector.h>
@@ -46,7 +50,7 @@
 #endif
 
 struct DMABufFeedback {
-    WTF_MAKE_STRUCT_FAST_ALLOCATED;
+    WTF_DEPRECATED_MAKE_STRUCT_FAST_ALLOCATED(DMABufFeedback);
 
     struct FormatTable {
         struct Data {
@@ -152,12 +156,12 @@ struct DMABufFeedback {
     }
 #endif
 
-    CString drmDevice() const
+    const dev_t* device() const
     {
 #if USE(LIBDRM)
-        return drmDeviceForUsage(&mainDevice, false);
+        return &mainDevice;
 #else
-        return { };
+        return nullptr;
 #endif
     }
 
@@ -182,12 +186,12 @@ struct DMABufFeedback {
 #endif
         }
 
-        CString drmDevice() const
+        const dev_t* device() const
         {
 #if USE(LIBDRM)
-            return drmDeviceForUsage(&targetDevice, flags & ZWP_LINUX_DMABUF_FEEDBACK_V1_TRANCHE_FLAGS_SCANOUT);
+            return &targetDevice;
 #else
-            return { };
+            return nullptr;
 #endif
         }
 
@@ -201,7 +205,7 @@ struct DMABufFeedback {
     std::pair<uint32_t, uint64_t> format(uint16_t index)
     {
         ASSERT(index < formatTable.size);
-        if (UNLIKELY(index >= formatTable.size))
+        if (index >= formatTable.size) [[unlikely]]
             return { 0, 0 };
 
         return { formatTable.data[index].format, formatTable.data[index].modifier };
@@ -224,6 +228,10 @@ struct _WPEToplevelWaylandPrivate {
     struct xdg_surface* xdgSurface;
     struct xdg_toplevel* xdgToplevel;
 
+#if USE(XDG_DECORATION_UNSTABLE_V1)
+    struct zxdg_toplevel_decoration_v1* xdgToplevelDecoration;
+#endif
+
     struct zwp_linux_dmabuf_feedback_v1* dmabufFeedback;
     struct zwp_linux_surface_synchronization_v1* surfaceSync;
     std::unique_ptr<DMABufFeedback> pendingDMABufFeedback;
@@ -557,6 +565,15 @@ static void wpeToplevelWaylandConstructed(GObject *object)
             const char* title = defaultTitle();
             xdg_toplevel_set_title(priv->xdgToplevel, title ? title : "");
             xdg_toplevel_set_app_id(priv->xdgToplevel, WTF::applicationID().data());
+#if USE(XDG_DECORATION_UNSTABLE_V1)
+            if (auto* xdgDecorationManager = wpeDisplayWaylandGetXDGDecorationManager(display)) {
+                // Even when asking for server-side decorations, the compositor may prefer
+                // client-side ones. This could be detected by attaching a listener, but that
+                // is pointless because painting client-side decorations is not supported.
+                priv->xdgToplevelDecoration = zxdg_decoration_manager_v1_get_toplevel_decoration(xdgDecorationManager, priv->xdgToplevel);
+                zxdg_toplevel_decoration_v1_set_mode(priv->xdgToplevelDecoration, ZXDG_TOPLEVEL_DECORATION_V1_MODE_SERVER_SIDE);
+            }
+#endif // USE(XDG_DECORATION_UNSTABLE_V1)
             wl_surface_commit(priv->wlSurface);
         }
     }
@@ -587,6 +604,11 @@ static void wpeToplevelWaylandDispose(GObject* object)
     auto* priv = WPE_TOPLEVEL_WAYLAND(object)->priv;
     priv->currentScreen = nullptr;
     priv->screens.clear();
+
+#if USE(XDG_DECORATION_UNSTABLE_V1)
+    g_clear_pointer(&priv->xdgToplevelDecoration, zxdg_toplevel_decoration_v1_destroy);
+#endif
+
     g_clear_pointer(&priv->xdgToplevel, xdg_toplevel_destroy);
     g_clear_pointer(&priv->dmabufFeedback, zwp_linux_dmabuf_feedback_v1_destroy);
     g_clear_pointer(&priv->surfaceSync, zwp_linux_surface_synchronization_v1_destroy);
@@ -654,6 +676,28 @@ static gboolean wpeToplevelWaylandSetMinimized(WPEToplevel* toplevel)
     return TRUE;
 }
 
+static GRefPtr<WPEDRMDevice> wpeToplevelWaylandGetDRMDevice(WPEToplevel* toplevel, const dev_t* device)
+{
+#if USE(LIBDRM)
+    auto* display = wpe_toplevel_get_display(toplevel);
+    auto* displayDevice = display ? wpe_display_get_drm_device(display) : nullptr;
+    drmDevicePtr drmDevice;
+    if (drmGetDeviceFromDevId(*device, 0, &drmDevice))
+        return nullptr;
+
+    if (!(drmDevice->available_nodes & (1 << DRM_NODE_PRIMARY)))
+        return nullptr;
+
+    if (displayDevice && !g_strcmp0(drmDevice->nodes[DRM_NODE_PRIMARY], wpe_drm_device_get_primary_node(displayDevice)))
+        return displayDevice;
+
+    return adoptGRef(wpe_drm_device_new(drmDevice->nodes[DRM_NODE_PRIMARY],
+        drmDevice->available_nodes & (1 << DRM_NODE_RENDER) ? drmDevice->nodes[DRM_NODE_RENDER] : nullptr));
+#else
+    return nullptr;
+#endif
+}
+
 static WPEBufferDMABufFormats* wpeToplevelWaylandGetPreferredDMABufFormats(WPEToplevel* toplevel)
 {
     auto* priv = WPE_TOPLEVEL_WAYLAND(toplevel)->priv;
@@ -663,16 +707,16 @@ static WPEBufferDMABufFormats* wpeToplevelWaylandGetPreferredDMABufFormats(WPETo
     if (!priv->committedDMABufFeedback)
         return nullptr;
 
-    auto mainDevice = priv->committedDMABufFeedback->drmDevice();
-    auto* builder = wpe_buffer_dma_buf_formats_builder_new(mainDevice.data());
+    auto mainDevice = wpeToplevelWaylandGetDRMDevice(toplevel, priv->committedDMABufFeedback->device());
+    auto* builder = wpe_buffer_dma_buf_formats_builder_new(mainDevice.get());
     for (const auto& tranche : priv->committedDMABufFeedback->tranches) {
         WPEBufferDMABufFormatUsage usage = tranche.flags & ZWP_LINUX_DMABUF_FEEDBACK_V1_TRANCHE_FLAGS_SCANOUT ? WPE_BUFFER_DMA_BUF_FORMAT_USAGE_SCANOUT : WPE_BUFFER_DMA_BUF_FORMAT_USAGE_RENDERING;
-        auto targetDevice = tranche.drmDevice();
-        wpe_buffer_dma_buf_formats_builder_append_group(builder, targetDevice.data(), usage);
+        auto targetDevice = wpeToplevelWaylandGetDRMDevice(toplevel, tranche.device());
+        wpe_buffer_dma_buf_formats_builder_append_group(builder, targetDevice.get(), usage);
 
         for (const auto& format : tranche.formats) {
             auto [fourcc, modifier] = priv->committedDMABufFeedback->format(format);
-            if (LIKELY(fourcc))
+            if (fourcc) [[likely]]
                 wpe_buffer_dma_buf_formats_builder_append_format(builder, fourcc, modifier);
         }
     }
@@ -755,7 +799,7 @@ static bool regionsEqual(WPERectangle* rectsA, unsigned rectsACount, WPERectangl
 void wpeToplevelWaylandSetOpaqueRectangles(WPEToplevelWayland* toplevel, WPERectangle* rects, unsigned rectsCount)
 {
     auto* priv = toplevel->priv;
-    if (regionsEqual(priv->opaqueRegion.rects.data(), priv->opaqueRegion.rects.size(), rects, rectsCount))
+    if (regionsEqual(priv->opaqueRegion.rects.mutableSpan().data(), priv->opaqueRegion.rects.size(), rects, rectsCount))
         return;
 
     priv->opaqueRegion.rects.clear();
@@ -882,14 +926,15 @@ struct zwp_linux_surface_synchronization_v1* wpeToplevelWaylandGetSurfaceSync(WP
 /**
  * wpe_toplevel_wayland_new:
  * @display: a #WPEDisplayWayland
+ * @max_views: the maximum number of views allowed, or 0 for no limit
  *
- * Create a new #WPEToplevel on @display.
+ * Create a new #WPEToplevel on @display with @max_views allowed
  *
  * Returns: (transfer full): a #WPEToplevel
  */
-WPEToplevel* wpe_toplevel_wayland_new(WPEDisplayWayland* display)
+WPEToplevel* wpe_toplevel_wayland_new(WPEDisplayWayland* display, guint maxViews)
 {
-    return WPE_TOPLEVEL(g_object_new(WPE_TYPE_TOPLEVEL_WAYLAND, "display", display, nullptr));
+    return WPE_TOPLEVEL(g_object_new(WPE_TYPE_TOPLEVEL_WAYLAND, "display", display, "max-views", maxViews, nullptr));
 }
 
 /**
