@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020-2021 Apple Inc. All rights reserved.
+ * Copyright (c) 2020-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -72,11 +72,15 @@ pas_enumerator* pas_enumerator_create(pas_root* remote_root_address,
                                       pas_enumerator_object_recording_mode record_object)
 {
     pas_enumerator* result;
-    uintptr_t* compact_heap_base;
-    size_t* compact_heap_size;
-    size_t* compact_heap_guard_size;
     pas_enumerator_region* region;
-    pas_heap_config** configs;
+
+    uintptr_t compact_heap_base;
+    size_t compact_heap_size;
+    size_t compact_heap_guard_size;
+    size_t compact_heap_reservation_bump;
+    void* compact_heap_copy_base;
+
+    const pas_heap_config* configs[pas_heap_config_kind_num_kinds];
     pas_heap_config_kind config_kind;
 
     region = NULL;
@@ -91,79 +95,104 @@ pas_enumerator* pas_enumerator_create(pas_root* remote_root_address,
     result->allocation_config.deallocate = deallocate;
     result->allocation_config.arg = result;
 
-    result->heap_config_datas = pas_enumerator_allocate(
-        result, sizeof(void*) * pas_heap_config_kind_num_kinds);
-    pas_zero_memory(result->heap_config_datas, sizeof(void*) * pas_heap_config_kind_num_kinds);
-
-    result->root = reader(result, remote_root_address, sizeof(pas_root), reader_arg);
-    if (!result->root)
-        goto fail;
-
-    PAS_ASSERT(result->root->magic == PAS_ROOT_MAGIC);
-    PAS_ASSERT(result->root->num_heap_configs == pas_heap_config_kind_num_kinds);
-
-    compact_heap_base = reader(
-        result, result->root->compact_heap_reservation_base, sizeof(uintptr_t), reader_arg);
-    if (!compact_heap_base)
-        goto fail;
-
-    compact_heap_size = reader(
-        result, result->root->compact_heap_reservation_size, sizeof(size_t), reader_arg);
-    if (!compact_heap_size)
-        goto fail;
-
-    compact_heap_guard_size = reader(
-        result, result->root->compact_heap_reservation_guard_size, sizeof(size_t), reader_arg);
-    if (!compact_heap_size)
-        goto fail;
-
-    result->compact_heap_remote_base = (void*)*compact_heap_base;
-    result->compact_heap_copy_base = (void*)(
-        (uintptr_t)reader(
-            result, (void*)(*compact_heap_base + *compact_heap_guard_size), *compact_heap_size,
-            reader_arg)
-        - *compact_heap_guard_size);
-    if (!result->compact_heap_copy_base)
-        goto fail;
-    
-    result->compact_heap_size = *compact_heap_size;
-    result->compact_heap_guard_size = *compact_heap_guard_size;
-
-    result->unaccounted_pages = pas_enumerator_allocate(result, sizeof(pas_ptr_hash_set));
-    pas_ptr_hash_set_construct(result->unaccounted_pages);
-
     result->reader = reader;
     result->reader_arg = reader_arg;
+    result->pinned_address = NULL;
     result->recorder = recorder;
     result->recorder_arg = recorder_arg;
     result->record_meta = record_meta;
     result->record_payload = record_payload;
     result->record_object = record_object;
+    result->lenient_compact_ptr_buffer = NULL;
+    result->lenient_compact_ptr_buffer_capacity = 0;
 
-    configs = reader(
-        result,
-        result->root->heap_configs,
-        sizeof(pas_heap_config*) * pas_heap_config_kind_num_kinds,
-        reader_arg);
-    if (!configs)
+    result->heap_config_datas = pas_enumerator_allocate(
+        result, sizeof(void*) * pas_heap_config_kind_num_kinds);
+    pas_zero_memory(result->heap_config_datas, sizeof(void*) * pas_heap_config_kind_num_kinds);
+
+    result->root = pas_enumerator_allocate(result, sizeof(pas_root));
+    if (!pas_enumerator_copy_remote(
+            result,
+            result->root,
+            remote_root_address,
+            sizeof(pas_root)))
         goto fail;
-    
+
+    PAS_ASSERT_WITH_DETAIL(result->root->magic == PAS_ROOT_MAGIC);
+    PAS_ASSERT_WITH_DETAIL(result->root->num_heap_configs == pas_heap_config_kind_num_kinds);
+
+    if (!pas_enumerator_copy_remote(
+            result,
+            &compact_heap_base,
+            result->root->compact_heap_reservation_base,
+            sizeof(uintptr_t)))
+        goto fail;
+
+    if (!pas_enumerator_copy_remote(
+            result,
+            &compact_heap_size,
+            result->root->compact_heap_reservation_size,
+            sizeof(size_t)))
+        goto fail;
+
+    if (!pas_enumerator_copy_remote(
+            result,
+            &compact_heap_guard_size,
+            result->root->compact_heap_reservation_guard_size,
+            sizeof(size_t)))
+        goto fail;
+
+    if (!pas_enumerator_copy_remote(
+            result,
+            &compact_heap_reservation_bump,
+            result->root->compact_heap_reservation_bump,
+            sizeof(size_t)))
+        goto fail;
+
+    if (!compact_heap_base)
+        goto fail;
+
+    PAS_ASSERT(compact_heap_reservation_bump >= compact_heap_guard_size);
+
+    compact_heap_copy_base = pas_enumerator_allocate(result, compact_heap_size);
+    if (!pas_enumerator_copy_remote(
+            result,
+            (void*)((uintptr_t)compact_heap_copy_base + compact_heap_guard_size),
+            (void*)(compact_heap_base + compact_heap_guard_size),
+            compact_heap_reservation_bump - compact_heap_guard_size))
+        goto fail;
+
+    result->compact_heap_size = compact_heap_size;
+    result->compact_heap_guard_size = compact_heap_guard_size;
+    result->compact_heap_remote_base = (void*)compact_heap_base;
+    result->compact_heap_copy_base = compact_heap_copy_base;
+
+    result->unaccounted_pages = pas_enumerator_allocate(result, sizeof(pas_ptr_hash_set));
+    pas_ptr_hash_set_construct(result->unaccounted_pages);
+
+    if (!pas_enumerator_copy_remote(
+            result,
+            configs,
+            result->root->heap_configs,
+            sizeof(const pas_heap_config*) * pas_heap_config_kind_num_kinds))
+        goto fail;
+
     for (PAS_EACH_HEAP_CONFIG_KIND(config_kind)) {
-        pas_heap_config* config;
-        pas_heap_config* remote_config;
+        const pas_heap_config* config;
+        const pas_heap_config* remote_config;
 
         if (config_kind == pas_heap_config_kind_null)
             continue;
 
         config = pas_heap_config_kind_get_config(config_kind);
 
-        PAS_ASSERT(config);
+        PAS_ASSERT_WITH_DETAIL(config);
 
-        remote_config = reader(result, configs[config->kind], sizeof(pas_heap_config), reader_arg);
+        remote_config = reader(result, (void*)(uintptr_t)configs[config->kind], sizeof(pas_heap_config), reader_arg);
         if (!remote_config)
             goto fail;
 
-        PAS_ASSERT(remote_config->kind == config->kind);
+        PAS_ASSERT_WITH_DETAIL(remote_config->kind == config->kind);
 
         if (!config->prepare_to_enumerate)
             continue;
@@ -191,35 +220,53 @@ void* pas_enumerator_allocate(pas_enumerator* enumerator,
     return pas_enumerator_region_allocate(&enumerator->region, size);
 }
 
+void* pas_enumerator_lenient_compact_ptr_buffer(pas_enumerator* enumerator,
+                                                size_t size)
+{
+    PAS_ASSERT(size);
+    if (size > enumerator->lenient_compact_ptr_buffer_capacity) {
+        size_t new_capacity;
+
+        new_capacity = PAS_MAX(2 * enumerator->lenient_compact_ptr_buffer_capacity, size);
+        enumerator->lenient_compact_ptr_buffer = pas_enumerator_allocate(enumerator, new_capacity);
+        enumerator->lenient_compact_ptr_buffer_capacity = new_capacity;
+    }
+    return enumerator->lenient_compact_ptr_buffer;
+}
+
 void* pas_enumerator_read_compact(pas_enumerator* enumerator,
                                   void* remote_address)
 {
     if ((uintptr_t)remote_address < (uintptr_t)PAS_INTERNAL_MIN_ALIGN)
         return remote_address;
     
-    PAS_ASSERT(remote_address >= (void*)((uintptr_t)enumerator->compact_heap_remote_base +
+    PAS_ASSERT_WITH_DETAIL(remote_address >= (void*)((uintptr_t)enumerator->compact_heap_remote_base +
                                          enumerator->compact_heap_guard_size));
-    PAS_ASSERT(remote_address < (void*)((uintptr_t)enumerator->compact_heap_remote_base +
+    PAS_ASSERT_WITH_DETAIL(remote_address < (void*)((uintptr_t)enumerator->compact_heap_remote_base +
                                         enumerator->compact_heap_size));
     return (void*)(
         (uintptr_t)enumerator->compact_heap_copy_base
         + (uintptr_t)remote_address - (uintptr_t)enumerator->compact_heap_remote_base);
 }
 
-void* pas_enumerator_read(pas_enumerator* enumerator,
-                          void* remote_address,
-                          size_t size)
+/* Call the "reader" to map remote_address into this process' address space.
+   The reader may invalidate previous mappings.
+ */
+static void* pas_enumerator_map_remote(pas_enumerator* enumerator,
+                                       void* remote_address,
+                                       size_t size)
 {
     void* compact_heap_end;
 
-    PAS_ASSERT(remote_address);
+    PAS_ASSERT(!enumerator->pinned_address);
+    PAS_ASSERT_WITH_DETAIL(remote_address);
 
     compact_heap_end = (void*)(
         (uintptr_t)enumerator->compact_heap_remote_base + enumerator->compact_heap_size);
     
     if (remote_address >= enumerator->compact_heap_remote_base
         && remote_address < compact_heap_end) {
-        PAS_ASSERT((uintptr_t)remote_address + size <= (uintptr_t)compact_heap_end);
+        PAS_ASSERT_WITH_DETAIL((uintptr_t)remote_address + size <= (uintptr_t)compact_heap_end);
         return pas_enumerator_read_compact(enumerator, remote_address);
     }
 
@@ -229,17 +276,50 @@ void* pas_enumerator_read(pas_enumerator* enumerator,
     return enumerator->reader(enumerator, remote_address, size, enumerator->reader_arg);
 }
 
+void* pas_enumerator_pin_remote(pas_enumerator* enumerator,
+                                void* remote_address,
+                                size_t size)
+{
+    void* mapped_address;
+
+    PAS_ASSERT(!enumerator->pinned_address);
+    mapped_address = pas_enumerator_map_remote(enumerator, remote_address, size);
+    enumerator->pinned_address = mapped_address;
+    return mapped_address;
+}
+
+void pas_enumerator_unpin_remote(pas_enumerator* enumerator,
+                                 void* pinned_address)
+{
+    PAS_ASSERT(enumerator->pinned_address == pinned_address);
+    enumerator->pinned_address = NULL;
+}
+
+bool pas_enumerator_copy_remote(pas_enumerator* enumerator,
+                                void* local_buffer,
+                                void* remote_address,
+                                size_t size)
+{
+    void* mapped_address;
+
+    mapped_address = pas_enumerator_map_remote(enumerator, remote_address, size);
+    if (!mapped_address)
+        return false;
+    memcpy(local_buffer, mapped_address, size);
+    return true;
+}
+
 void pas_enumerator_add_unaccounted_pages(pas_enumerator* enumerator,
                                           void* remote_address,
                                           size_t size)
 {
     size_t offset;
 
-    PAS_ASSERT(pas_is_aligned((uintptr_t)remote_address, enumerator->root->page_malloc_alignment));
-    PAS_ASSERT(pas_is_aligned(size, enumerator->root->page_malloc_alignment));
+    PAS_ASSERT_WITH_DETAIL(pas_is_aligned((uintptr_t)remote_address, enumerator->root->page_malloc_alignment));
+    PAS_ASSERT_WITH_DETAIL(pas_is_aligned(size, enumerator->root->page_malloc_alignment));
 
     /* Catch bogus sizes, in case we did some overflow or weird subtraction. */
-    PAS_ASSERT((uint64_t)size < ((uint64_t)1 << PAS_ADDRESS_BITS));
+    PAS_ASSERT_WITH_DETAIL((uint64_t)size < ((uint64_t)1 << PAS_ADDRESS_BITS));
 
     for (offset = 0; offset < size; offset += enumerator->root->page_malloc_alignment) {
         pas_ptr_hash_set_set(enumerator->unaccounted_pages,
@@ -253,7 +333,7 @@ bool pas_enumerator_exclude_accounted_page(pas_enumerator* enumerator,
 {
     static const bool verbose = false;
     bool result;
-    PAS_ASSERT(pas_is_aligned((uintptr_t)remote_address, enumerator->root->page_malloc_alignment));
+    PAS_ASSERT_WITH_DETAIL(pas_is_aligned((uintptr_t)remote_address, enumerator->root->page_malloc_alignment));
     result = pas_ptr_hash_set_remove(
         enumerator->unaccounted_pages, remote_address, NULL, &enumerator->allocation_config);
     if (verbose) {
@@ -271,11 +351,11 @@ void pas_enumerator_exclude_accounted_pages(pas_enumerator* enumerator,
 {
     size_t offset;
 
-    PAS_ASSERT(pas_is_aligned((uintptr_t)remote_address, enumerator->root->page_malloc_alignment));
-    PAS_ASSERT(pas_is_aligned(size, enumerator->root->page_malloc_alignment));
+    PAS_ASSERT_WITH_DETAIL(pas_is_aligned((uintptr_t)remote_address, enumerator->root->page_malloc_alignment));
+    PAS_ASSERT_WITH_DETAIL(pas_is_aligned(size, enumerator->root->page_malloc_alignment));
 
     /* Catch bogus sizes, in case we did some overflow or weird subtraction. */
-    PAS_ASSERT((uint64_t)size < ((uint64_t)1 << PAS_ADDRESS_BITS));
+    PAS_ASSERT_WITH_DETAIL((uint64_t)size < ((uint64_t)1 << PAS_ADDRESS_BITS));
 
     for (offset = 0; offset < size; offset += enumerator->root->page_malloc_alignment)
         pas_enumerator_exclude_accounted_page(enumerator, (void*)((uintptr_t)remote_address + offset));
@@ -290,7 +370,7 @@ void pas_enumerator_record(pas_enumerator* enumerator,
         return;
 
     /* Catch bogus sizes, in case we did some overflow or weird subtraction. */
-    PAS_ASSERT((uint64_t)size < ((uint64_t)1 << PAS_ADDRESS_BITS));
+    PAS_ASSERT_WITH_DETAIL((uint64_t)size < ((uint64_t)1 << PAS_ADDRESS_BITS));
     
     switch (kind) {
     case pas_enumerator_meta_record:
@@ -328,9 +408,9 @@ void pas_enumerator_record_page_payload_and_meta(pas_enumerator* enumerator,
                                                  uintptr_t payload_begin,
                                                  uintptr_t payload_end)
 {
-    PAS_ASSERT(payload_begin < page_size);
-    PAS_ASSERT(payload_end <= page_size);
-    PAS_ASSERT(payload_begin < payload_end);
+    PAS_ASSERT_WITH_DETAIL(payload_begin < page_size);
+    PAS_ASSERT_WITH_DETAIL(payload_end <= page_size);
+    PAS_ASSERT_WITH_DETAIL(payload_begin < payload_end);
 
     /* We assume, correctly for now, that non-payload areas of the page are always committed if the page
        is committed. */
@@ -345,7 +425,7 @@ void pas_enumerator_record_page_payload_and_meta(pas_enumerator* enumerator,
 
     if (enumerator->record_payload) {
         if (page_size == granule_size) {
-            PAS_ASSERT(!use_counts);
+            PAS_ASSERT_WITH_DETAIL(!use_counts);
             pas_enumerator_record(enumerator,
                                   (void*)(page_boundary + payload_begin),
                                   payload_end - payload_begin,
@@ -354,8 +434,8 @@ void pas_enumerator_record_page_payload_and_meta(pas_enumerator* enumerator,
             uintptr_t granule_index;
             pas_range span;
 
-            PAS_ASSERT(page_size > granule_size);
-            PAS_ASSERT(use_counts);
+            PAS_ASSERT_WITH_DETAIL(page_size > granule_size);
+            PAS_ASSERT_WITH_DETAIL(use_counts);
 
             span = pas_range_create(payload_begin, payload_begin);
             
@@ -390,16 +470,16 @@ bool pas_enumerator_for_each_heap(pas_enumerator* enumerator,
 {
     size_t index;
     pas_heap* heap;
-    pas_heap** first_heap;
+    pas_heap* first_heap;
     pas_heap** static_heaps;
 
-    first_heap = pas_enumerator_read(enumerator,
-                                     enumerator->root->all_heaps_first_heap,
-                                     sizeof(pas_heap*));
-    if (!first_heap)
+    if (!pas_enumerator_copy_remote(enumerator,
+                                    &first_heap,
+                                    enumerator->root->all_heaps_first_heap,
+                                    sizeof(pas_heap*)))
         return false;
 
-    for (heap = pas_enumerator_read_compact(enumerator, *first_heap);
+    for (heap = pas_enumerator_read_compact(enumerator, first_heap);
          heap;
          heap = pas_compact_heap_ptr_load_remote(enumerator,
                                                  &heap->next_heap)) {
@@ -407,20 +487,18 @@ bool pas_enumerator_for_each_heap(pas_enumerator* enumerator,
             return false;
     }
 
-    static_heaps = pas_enumerator_read(enumerator,
-                                       enumerator->root->static_heaps,
-                                       sizeof(pas_heap*) * enumerator->root->num_static_heaps);
+    static_heaps = pas_enumerator_read_compact(enumerator,
+                                               enumerator->root->static_heaps);
     if (!static_heaps)
         return false;
 
     for (index = enumerator->root->num_static_heaps; index--;) {
-        pas_heap* heap;
+        pas_heap heap;
 
-        heap = pas_enumerator_read(enumerator, static_heaps[index], sizeof(pas_heap));
-        if (!heap)
+        if (!pas_enumerator_copy_remote(enumerator, &heap, static_heaps[index], sizeof(pas_heap)))
             return false;
-        
-        if (!callback(enumerator, heap, arg))
+
+        if (!callback(enumerator, &heap, arg))
             return false;
     }
 
