@@ -22,3 +22,6 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+#import "../../TestPDFDocument.h"
+#import <pal/spi/cg/CoreGraphicsSPI.h>
