@@ -31,6 +31,10 @@
 #include <WebKit/WKRetainPtr.h>
 #include <WebKit/WKView.h>
 
+#if ENABLE(WPE_PLATFORM)
+#include <wpe/headless/wpe-headless.h>
+#endif
+
 namespace TestWebKitAPI {
 
 PlatformWebView::PlatformWebView(WKContextRef contextRef)
@@ -67,13 +71,29 @@ PlatformWebView::PlatformWebView(WKPageRef relatedPage)
 PlatformWebView::~PlatformWebView()
 {
     WKRelease(m_view);
-    delete m_window;
+#if ENABLE(WPE_PLATFORM)
+    if (g_type_class_peek(WPE_TYPE_DISPLAY))
+        return;
+#endif
+    delete static_cast<WPEToolingBackends::HeadlessViewBackend*>(m_window);
 }
 
 void PlatformWebView::initialize(WKPageConfigurationRef configuration)
 {
+#if ENABLE(WPE_PLATFORM)
+    if (g_type_class_peek(WPE_TYPE_DISPLAY)) {
+        auto* display = wpe_display_get_primary();
+        g_assert(WPE_IS_DISPLAY_HEADLESS(display));
+        m_view = WKViewCreate(display, configuration);
+        auto* wpeView = WKViewGetView(m_view);
+        wpe_view_focus_in(wpeView);
+        m_window = wpe_view_get_toplevel(wpeView);
+        wpe_toplevel_resize(WPE_TOPLEVEL(m_window), 800, 600);
+        return;
+    }
+#endif
     m_window = new WPEToolingBackends::HeadlessViewBackend(800, 600);
-    m_view = WKViewCreateDeprecated(m_window->backend(), configuration);
+    m_view = WKViewCreateDeprecated(static_cast<WPEToolingBackends::HeadlessViewBackend*>(m_window)->backend(), configuration);
 }
 
 WKPageRef PlatformWebView::page() const
